/* 	CoffeeBreak.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 25-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * 25-02-2008	dennis@brains2b.nl		Changed close operation
 * 16-11-2008	dennis@brains2b.nl		Added Aboutbox
 */
package org.brains2b.coffee;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.UIManager;

import org.brains2b.coffee.decode.EncodeHandler;
import org.brains2b.coffee.file.ConfigurationHandler;
import org.brains2b.coffee.file.FileConnector;
import org.brains2b.coffee.gui.CfActionMap;
import org.brains2b.coffee.gui.CfFrame;
import org.brains2b.coffee.gui.binary.BinaryOptions;
import org.brains2b.coffee.gui.settings.CfCoffeeList;
import org.brains2b.coffee.gui.settings.CfIdentity;
import org.brains2b.coffee.gui.settings.CfOptions;
import org.brains2b.coffee.gui.settings.CfServer;
import org.brains2b.coffee.plugin.MimeHandler;
import org.brains2b.coffee.plugin.nzb.NzbGuiPanel;
import org.brains2b.coffee.plugin.search.NewsSearch;
import org.brains2b.console.ArgumentMap;
import org.brains2b.console.Main;
import org.brains2b.log.Logger;
import org.brains2b.swingext.conf.UserPreference;
import org.brains2b.swingext.dialog.AboutPane;
import org.brains2b.swingext.dialog.DialogHelper;
import org.brains2b.swingext.preference.PreferencePane;

/** Main class for CoffeeBreak
 *  <UL>Arguments passed to main
 *  <li>%1 - path to different coffeebreak directory</li></ul>
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version $Revision: 1.14 $ $Date: 2010/12/30 04:38:29 $
 */
public class CoffeeBreak extends Main implements ActionListener {

   private class WindowHandler extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            close();
        }
        
    }

   	private CfFrame m_frame;
   	
    /**
     * Default constructor
     */
    public CoffeeBreak() {
        super();
        init();
        loadUIManagerChanges();
    }

    public static void main(String[] args) {
		CoffeeBreak cb=new CoffeeBreak();
		cb.makeArgumentList(args);
		cb.doit();
    }
    
    public void actionPerformed(ActionEvent e) {
    	String actionCommand = e.getActionCommand();
        if ("System.Exit".equals(actionCommand)) {
            close();
        } else if ("System.Settings".equals(actionCommand)) {
            showSettings();
        } else if ("System.About".equals(actionCommand)) {
        	showAbout();
        }

    }
    
    private boolean checkSystemPath() {
    	if (getArgument("data")!=null) {
    		FileConnector.setSystemPath(getArgument("data"));
    		return true;
    	}
    	UserPreference pref = new UserPreference("coffeebreak.properties");
    	pref.load(); 
    	if (pref.getProperty("system.path")!=null) {
    		FileConnector.setSystemPath(pref.getProperty("system.path"));
    		return true;
    	}
    	
    	
    	return false;
    	
    }
    
    private void showSettings() {
        CfOptions options = new CfOptions();
        options.addOptionView(new CfCoffeeList("server",new CfServer()));
        options.addOptionView(new CfCoffeeList("identity",new CfIdentity()));
        options.addOptionView(new BinaryOptions());
        DialogHelper.showDialog(m_frame, options, "Settings");
    }
    
    private void init() {
    	// TODO  Implement runtime path here 
        ArgumentMap map=new ArgumentMap();
        map.addArgument("data", false);
        setArgumentMap(map);
    }
    
    private void showAbout() {
    	AboutPane about = new AboutPane();
    	about.setAuthor(getClass().getPackage().getSpecificationVendor());
    	about.setVersion(getClass().getPackage().getSpecificationVersion());
    	about.setTitle(getClass().getPackage().getSpecificationTitle());
    	about.setDate(getClass().getPackage().getImplementationVersion());
    	about.setLogo("/org/brains2b/coffee/resource/News32.gif");
		try {
    		about.setTextfromFile("/license.txt");
		} catch (IOException iex) {}
    	DialogHelper.showDialog(m_frame, about, "About");
    }
    
    private void doit() {
    	
    	m_frame=new CfFrame(new CfActionMap(this));
    	String title = "CoffeeBreak (dev)";
    	if (getClass().getPackage().getSpecificationTitle()!=null) {
    		title = getClass().getPackage().getSpecificationTitle() + " " + getClass().getPackage().getSpecificationVersion();
    	}
    	m_frame.setTitle(title);
        m_frame.addWindowListener(new WindowHandler());
        m_frame.setSize(800,600);
        
        boolean b = checkSystemPath();
        
        TicketBroker broker=TicketBroker.getBroker();
        broker.registerHandler(new FileConnector());
        broker.registerHandler(new ConfigurationHandler());
        broker.registerHandler(new EncodeHandler());
        /*Timer t=new Timer(true);
        t.schedule(broker,300,300);*/
        
        MimeHandler mh= new MimeHandler(m_frame);
        mh.registerPlugin(new NzbGuiPanel());
        mh.registerPlugin(new NewsSearch());
        broker.registerHandler(mh);
        
        broker.addTaskListener(m_frame);
        m_frame.setVisible(true);
        if (!b) {
        	initialize();
        }
    }
        
    private void close() {
        m_frame.setVisible(false);
    	try {
            TicketBroker.getBroker().close();
        } catch (Exception ex) {
            Logger.printException(ex);
        }
        m_frame.dispose();
        
    }
   
    private void initialize() {
    	UserPreference pref = new UserPreference("coffeebreak.properties");
    	File dflt = new File((String) null,"coffeebreak");
    	pref.setProperty("system.path", dflt.getAbsolutePath());
    	PreferencePane pp = new PreferencePane(pref);
    	pp.setPropertyType("system.path", PreferencePane.FILE);
    	DialogHelper.showDialog(m_frame, pp , "Select location for the coffeebreak data");
    	FileConnector.setSystemPath(pref.getProperty("system.path"));
    	pref.store();
    	showSettings();
    }
    private void loadUIManagerChanges() {
    	final Color intOrange = new Color(217,72,3);
    	final Color intOrangeAlpha = new Color(217,72,3,96);
    	UIManager.put("ProgressBar.foreground", intOrange);
    	UIManager.put("ComboBox.selectionBackground", intOrangeAlpha);
    	UIManager.put("List.selectionBackground", intOrangeAlpha);
    	UIManager.put("MenuItem.selectionBackground",new Color(219,163,138));
    	UIManager.put("ScrollBar.thumb", intOrangeAlpha);
    	UIManager.put("ScrollBar.thumbShadow", intOrange);
    	UIManager.put("TabbedPane.selected", intOrangeAlpha);
    	UIManager.put("Table.selectionBackground", intOrangeAlpha);
    	UIManager.put("Tree.selectionBackground", intOrangeAlpha);
    	UIManager.put("TextField.selectionBackground", intOrangeAlpha);
    	UIManager.put("TextPane.selectionBackground", intOrangeAlpha);
    	UIManager.put("ProgressBar.selectionForeground",Color.white);
    	UIManager.put("ProgressBar.selectionBackground",Color.darkGray);
    	UIManager.put("ProgressBar.foreground",intOrange);
    }
}
