/* 	TicketQueue.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 8-aug-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Jan 6, 2009	dennis@brains2b.nl		Only clear a synchronized list		
 */
package org.brains2b.coffee;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.message.InfoMessage;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.monitor.ProgressMonitor;
import org.brains2b.coffee.plugin.CoffeeExchangeControl;

/**
 * [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version $Revision: 1.7 $ $Date: 2010/07/05 00:31:15 $
 */
public class TicketQueue implements Comparator, TicketHandler {

    private List m_list;
    private int m_sequence=0;
    
    public TicketQueue() {
        m_list= Collections.synchronizedList(new ArrayList());
    }
    
    public int compare(Object o1, Object o2) {
        int result=0;
        if (o1 instanceof Ticket && o2 instanceof Ticket) {
             result=((Ticket) o1).getSequence()-((Ticket) o2).getSequence();
        }
        if (result==0) {
            result=1;
        }
        return result;
    }
    
    public synchronized void add(Ticket t) {
        if (t.getEvent().getSource() instanceof CoffeeExchangeControl) {
            t.setSequence(t.getEvent().getID()+(m_sequence++)+1000000);
        } else {
            t.setSequence(t.getEvent().getID() + m_sequence++);
        }
        m_list.add(t);
        Collections.sort(m_list,this);
        
    }
    
    public boolean isEmpty() {
        return m_list.isEmpty();
    }
    
    public synchronized Iterator iterator() {
        return m_list.iterator();
    }
    
    public int getSize() {
        return m_list.size();
    }
    
    public void clear() {
        synchronized (m_list) {
    		m_list.clear();
        }
    }
    
    public Ticket get(int index) {
    	return (Ticket) m_list.get(index);
    }

	public void addProgressMonitor(ProgressMonitor m) {
	}

	public void close() throws Exception {
		
	}

	public int[] getHandledEvents() {
		return new int[] { CoffeeEvent.QUEUE_DOWN,
				CoffeeEvent.QUEUE_UP,
				CoffeeEvent.QUEUE_PAUSE,
				CoffeeEvent.QUEUE_RESUME,
				CoffeeEvent.QUEUE_REMOVE
		};
	}

	public NNTPMessage handleEvent(CoffeeEvent event) throws CoffeeSpill {
		switch (event.getID()) {
			case CoffeeEvent.QUEUE_DOWN:
				Ticket[] tickets = (Ticket[]) event.getArgument();
				synchronized (m_list) {
					int idx = m_list.indexOf(tickets[tickets.length-1]);
					if (idx==-1 || idx>=m_list.size()-1) return null;
					int seq = ((Ticket) m_list.get(idx+1)).getSequence();
					int last = seq;
					for (int i=0;i<tickets.length;i++) {
						last=tickets[i].getSequence();
						tickets[i].setSequence(seq);
						seq=last;
					}
					((Ticket) m_list.get(idx+1)).setSequence(seq);
				}
				break;
			case CoffeeEvent.QUEUE_UP:
				tickets = (Ticket[]) event.getArgument();
				synchronized (m_list) {
					int idx = m_list.indexOf(tickets[tickets.length-1]);
					if (idx<=0) return null;
					int seq = ((Ticket) m_list.get(idx-1)).getSequence();
					int last = seq;
					for (int i=0;i<tickets.length;i++) {
						last=tickets[i].getSequence();
						tickets[i].setSequence(seq);
						seq=last;
					}
					((Ticket) m_list.get(idx-1)).setSequence(seq);
				}
				break;
			case CoffeeEvent.QUEUE_PAUSE:
				if (event.getArgument() instanceof List) {
					for (Iterator it= iterator();it.hasNext();) {
						((Ticket) it.next()).setWait(true);
					}
				} else {
					tickets = (Ticket[]) event.getArgument();
					for (int i=0;i<tickets.length;i++) {
						tickets[i].setWait(true);
					}
				}
				break;
			case CoffeeEvent.QUEUE_RESUME:
				if (event.getArgument() instanceof List) {
					for (Iterator it= iterator();it.hasNext();) {
						((Ticket) it.next()).setWait(false);
					}
				} else {
					tickets = (Ticket[]) event.getArgument();
					for (int i=0;i<tickets.length;i++) {
						tickets[i].setWait(false);
					}
				}
				break;
			case CoffeeEvent.QUEUE_REMOVE:
				if (event.getArgument() instanceof List) {
					clear();
				} else {
					tickets = (Ticket[]) event.getArgument();
					for (int i=0;i<tickets.length;i++) {
						tickets[i].setFinished(true);
						tickets[i].setResult(Ticket.RESULT_REMOVE);
						
					}
				}
				break;
		}
		return new InfoMessage();
	}

	public boolean isAvailable() {
		return true;
	}

	public void open() throws Exception {
		
	}

	public void removeProgressMonitor(ProgressMonitor m) {
	}

	public List getList() {
		return new ArrayList(m_list);
	}
}
