package org.brains2b.coffee;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.brains2b.coffee.plugin.CoffeeExchangeControl;
import org.brains2b.task.TaskListener;

public class TicketThread extends Thread {

	private TicketQueue m_queue;
	private boolean m_closed;
	private List m_listeners;
	private Ticket m_current;
	
	public TicketThread(TicketQueue queue) {
		super();
		m_queue = queue;
	}
	
	
	public void run() {
		while(true) {
			Ticket t = null;
			synchronized (m_queue) {
				Iterator it = m_queue.iterator();
    			if (it.hasNext()) {
    				t = (Ticket) it.next();
    				while (it.hasNext() &&  t.isWait()) {
    					t = (Ticket) it.next();
    				}
    				if (t.getTicketHandler().isAvailable()) {
			   			if (!t.isFinished() && t.getResult()!=Ticket.RESULT_REMOVE) {
							it.remove();
			   			} else {
			   				it.remove();
			   				t =null;
			   			}
    				} else {
    					t=null;
    				}
			   		 
    			}
			}
			
            if (t!=null) {
				startTicket(t);
            }
    		
			if (m_queue.isEmpty()) {
				if (isClosed()) return;
				try {
					sleep(200);
				} catch (Throwable ex){}
			}
		}
	}


	public boolean isClosed() {
		return m_closed;
	}


	public void setClosed(boolean closed) {
		m_closed = closed;
	}
	
	private void startTicket(Ticket ticket) {
		m_current = ticket;
    	//final int eventId = ticket.getEvent().getID();
        if (ticket.getEvent().getSource() instanceof CoffeeExchangeControl) {
            CoffeeExchangeControl cec= (CoffeeExchangeControl) ticket.getEvent().getSource();
            ticket.addTaskListener(cec.getTaskListener());
           
        } 
        if (m_listeners != null) {
            for (int i = 0; i < m_listeners.size(); i++) {
                ticket.addTaskListener((TaskListener) m_listeners.get(i));
            }
        }
       
       ticket.run();
    }


	public void addTaskListener(TaskListener listener) {
	    if (m_listeners == null) {
	        m_listeners = new ArrayList();
	    }
	    m_listeners.add(listener);
	}


	public void removeTaskListener(TaskListener listener) {
	    if (m_listeners != null) {
	        m_listeners.remove(listener);
	    }
	}


	public Ticket getCurrent() {
		return m_current;
	}
}
