package org.brains2b.coffee.decode;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;

import org.brains2b.coffee.CoffeeSpill;
import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.TicketHandler;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.ArticleBody;
import org.brains2b.coffee.message.ArticleHeader;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.local.FileListMessage;
import org.brains2b.coffee.message.local.FileMessage;
import org.brains2b.coffee.message.local.Options;
import org.brains2b.coffee.monitor.ProgressMonitor;
import org.brains2b.coffee.plugin.CoffeeExchangeControl;
import org.brains2b.coffee.util.SubjectHelper;
import org.brains2b.task.Task;
import org.brains2b.task.TaskEvent;
import org.brains2b.task.TaskListener;

public class EncodeHandler implements TicketHandler, CoffeeExchange {

	public class PosterParent implements CoffeeExchangeControl, TaskListener {

		private Article m_article;
		private int m_result = 0;

		public PosterParent(Article article) {
			super();
			m_article = article;
		}

		public TaskListener getTaskListener() {
			return this;
		}

		public Stack getMessages() {
			return null;
		}

		public int getResult() {
			return m_result;
		}

		public boolean isFinished() {
			return m_finished;
		}

		public void setFinished(boolean b) {
			m_finished = b;
		}

		public void run() {
			setFinished(false);
			try {
				FileListMessage fm = (FileListMessage) m_article.getFile();
				Options options = fm.getOptions() == null ? m_defaultOptions
						: fm.getOptions();
				final long blocksize = 1024L * (long) Integer.parseInt(options
						.getString("block-size"));
				int fileCnt = 0;
				for (Iterator it = fm.getFiles().iterator(); it.hasNext();) {
					File f = (File) it.next();
					fileCnt++;
					if (f.length() > blocksize) {
						final int totalParts = (int) (f.length() / blocksize + 1);
						for (int i = 0; i < totalParts; i++) {
							Article a = new Article();
							a.setHeader((ArticleHeader) m_article.getHeader()
									.clone());
							ArticleBody body = new ArticleBody();
							a.setBody(body);
							if (i == 0 && m_article.getBody() != null) {
								body.addLine(m_article.getBody().getContent());
							}
							a.setPart(i + 1);
							a.setTotalParts(totalParts);
							FileMessage nfm = new FileMessage(f, options
									.getString("encoding"));
							nfm.setBlockSize((int) blocksize);
							a.setFile(nfm);
							a.setSubject(SubjectHelper.createSubject(a,
									options, fileCnt, fm.getFiles().size()));
							fireCoffeeEvent(this, CoffeeEvent.ENCODE_POST, a);
						}
					} else {
						Article a = (Article) m_article.clone();
						a.setPart(1);
						a.setTotalParts(1);
						FileMessage nfm = new FileMessage(f, options
								.getString("encoding"));
						nfm.setBlockSize((int) blocksize);
						a.setFile(nfm);
						a.setSubject(SubjectHelper.createSubject(a, options,
								fileCnt, fm.getFiles().size()));
						fireCoffeeEvent(this, CoffeeEvent.ENCODE_POST, a);
					}
				}
				m_result = Task.RESULT_OK;
			} catch (Exception ex) {
				m_result = Task.RESULT_ERROR;
			} finally {
				setFinished(true);
			}

		}

		public void setCurrentMessage(NNTPMessage message) {

		}

		public void taskProgressed(TaskEvent e) {

		}
	}

	private Options m_defaultOptions;
	private boolean m_finished = true;
	private int m_encodersRunning;
    private List m_monitors;
	
	
	public void addProgressMonitor(ProgressMonitor m) {
		if (m_monitors==null) {
			m_monitors = new ArrayList();
		}
		m_monitors.add(m);
	}

	public void close() throws Exception {
		
	}

	public int[] getHandledEvents() {
		return new int[] { 
			CoffeeEvent.ENCODE_ARTICLE, CoffeeEvent.ENCODE_POST,
			CoffeeEvent.CALCULATE_POSTS
		};
	}

	public NNTPMessage handleEvent(CoffeeEvent event) throws CoffeeSpill {
		Article article = (Article) event.getArgument();
		switch (event.getID()) {
		case CoffeeEvent.ENCODE_ARTICLE:
			TicketBroker.getBroker().broker(
					new CoffeeEvent(this, CoffeeEvent.PROCESS,
							new PosterParent(article)));
			break;
		case CoffeeEvent.ENCODE_POST:
			try {
				m_encodersRunning++;
				Encoder enc = DecoderFactory.createEncoder(article.getFile()
						.getMimeType());
				String content = enc.encode(article.getFile().getFile(),
						article.getPart(), article.getFile().getBlockSize());
				article.getBody().addLine(content);
				return article;
			} finally {
				m_encodersRunning--;
			}
		case CoffeeEvent.CALCULATE_POSTS:
			FileListMessage fm = (FileListMessage) article.getFile();
			Options options = fm.getOptions() == null ? m_defaultOptions
						: fm.getOptions();
			final long blocksize = 1024L * (long) Integer.parseInt(options
						.getString("block-size"));
			int parts = 0;
			for (Iterator it = fm.getFiles().iterator();it.hasNext();) {
				File f = (File) it.next();
				parts += (int) (f.length()/blocksize)+1;
			}
			article.setTotalParts(parts);
			return article;
		}
		return null;
	}

	public synchronized boolean isAvailable() {
		return m_encodersRunning < 6;
	}

	public void open() throws Exception {
		fireCoffeeEvent(this, CoffeeEvent.GET_LIST, "options/binary");

	}

	public void removeProgressMonitor(ProgressMonitor m) {
		if (m_monitors!=null) {
			m_monitors.remove(m);
		}

	}

	public void setCurrentMessage(NNTPMessage message) {
		if (message instanceof Options) {
			m_defaultOptions = (Options) message;
		}
	}

	public void fireCoffeeEvent(CoffeeExchange source, int action,
			Object argument) {
		TicketBroker.getBroker().broker(
				new CoffeeEvent(source, action, argument));
	}

}
