/* 	CoffeeEvent.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 25-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * 25-02-2008	dennis@brains2b.org		Implemented action GET_ARTICLE_LIST for reading complete list		
 */
package org.brains2b.coffee.event;

import java.awt.AWTEvent;

/** Default Event for CoffeeBreak
 * <p>Defines the default actions for CoffeeBreak
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version $Revision: 1.8 $ $Date: 2010/07/05 00:32:15 $
 */
public class CoffeeEvent extends AWTEvent {
    
	public final static int NONE=0;
	
    public final static int CHANGE_SERVER=1;
    public final static int XML_READ_DIRECT=   10;
    public final static int USE =100;
    
    public final static int XML_READ= 	1000;
    public final static int PROCESS	= 	1999;
    
    public final static int QUEUE_REMOVE= 1100;
    public final static int QUEUE_UP= 1102;
    public final static int QUEUE_DOWN= 1101;
    public final static int QUEUE_PAUSE= 1103;
    public final static int QUEUE_RESUME= 1104;
    
    public final static int CALCULATE_POSTS=2100;
    
    public final static int NEWS_GROUP= 7010;
    public final static int XML_SAVE  = 7100;
    public final static int XML_SAVE_QUIET = 7102;
    public final static int SAVE  = 7101;
    public final static int XML_READ_CONFIG  = 1002;
    public final static int XML_SAVE_CONFIG  = 7112;
    
    public final static int ENCODE_POST =7501;
    public final static int ENCODE_ARTICLE =7502;
    
    public final static int GET_GROUPLIST = 8000;
    public final static int GET_ARTICLES = 8015;
    public final static int GET_ARTICLE_LIST = 8016;
    public final static int GET_ARTICLE = 8020;
    public final static int GET_ATTACHMENT = 8021;
    public final static int SAVE_ATTACHMENT = 8022;
    public final static int SAVE_POST =8023;
    public final static int POST_ARTICLE = 8051;
    
    public final static int GET_LIST =6000;
    public final static int LIST_ADD =6010;
    public final static int LIST_REMOVE =6011;
    public final static int EXPIRE_SERVER = 6099;
    public final static int SUBSCRIPTION_GROUP=6500;
    
    public final static int FILTER  =   9010;
    public final static int GROUP_ARTICLES = 9012;
    
    private Object m_argument;
    
    /**
     * Default Constructor
     * @param source CoffeeExchange, the source for this Event
     * @param action int, an action as defined as a constant, or an id > 10000
     * @param argument Object, argument to pass to this event
     */
    public CoffeeEvent(CoffeeExchange source, int action, Object argument) {
        super(source,action);
        m_argument=argument;
    }
    
    /**
     * get the argument for this Event 
     * @return Object, the argument
     */
    public Object getArgument() {
        return m_argument;
    }
    
    /**
     * get the source of this event
     * <p>conviniance method for (CoffeeExchange) getSource()
     * @return the source for this Event
     * @see #getSource()
     */
    public CoffeeExchange getExchange() {
        return (CoffeeExchange) getSource();
    }
    
    /**
     * two CoffeeEvents are considered equal if 
     * the id, source and argument match
     * @return true, if CoffeeEvents are equal
     */
    public boolean equals(Object obj) {
        if (!(obj instanceof CoffeeEvent)) return false;
        CoffeeEvent e = (CoffeeEvent) obj;
        if (e.getID()==getID() && e.getSource()==getSource() && e.getArgument()==e.getArgument()) {
            return true;
        }
        return false;
    }
}
