/* 	ArticleHandler.java
 *	Copyright (c) 2007-2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Sept 4, 2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 25, 2008	dennis@brains2b.nl		Use String.valueOf(byte[],int,int) instead of new String()
 * Apr 09, 2008 dennis@brains2b.nl		Use decode on read content		
*/
package org.brains2b.coffee.file;

import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.ArticleBody;
import org.brains2b.coffee.message.ArticleHeader;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.util.XmlHelper;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/** ContentHandler for Article information on file
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version $Revision: 1.3 $ $Date: 2010/07/05 00:35:10 $
 */
public class ArticleHandler extends FileContentHandler {

	private String m_lastKey;
	private Article m_article;
	
	public NNTPMessage getMessage() {
		return m_article;
	}

	public void characters(char[] ch, int start, int length)
			throws SAXException {
		String s = String.valueOf(ch,start,length);
		if (!m_lastKey.equals("body")) {
			m_article.getHeader().addHeaderValue(m_lastKey, XmlHelper.decode(s));
		} else {
			m_article.getBody().addLine(s);
		}

	}

	public void endDocument() throws SAXException {
		
	}

	public void endElement(String uri, String localName, String qName)
			throws SAXException {
		m_lastKey=null;

	}

	public void startDocument() throws SAXException {
		
	}

	public void startElement(String uri, String localName, String qName,
			Attributes atts) throws SAXException {
		m_lastKey= qName;
		if ("article".equals(qName)) {
			m_article = new Article();
			m_article.setId(atts.getValue("id"));
			m_article.setNumber(Integer.parseInt(atts.getValue("number")));
			m_article.setSaved(Boolean.parseBoolean(atts.getValue("saved")));
		} else if ("header".equals(qName)) {
			m_article.setHeader(new ArticleHeader());
		} else if ("body".equals(qName)) {
			m_article.setBody(new ArticleBody());
		}
	}

}
