/* 	NewsGroupHandler.java
 *	Copyright (c) 2005-2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 23-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 25, 2008	dennis@brains2b.nl		Cleaned up, Changes for NewsGroup becoming an interface
 * 										Use String.valueOf(byte[],int,int) instead of new String()
 * 										Use DateHelper.isoDates				
*/
package org.brains2b.coffee.file;

import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.ArticleHeader;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.NewsGroup;
import org.brains2b.coffee.message.NewsGroupImpl;
import org.brains2b.util.DateHelper;
import org.brains2b.util.XmlHelper;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/** ContentHandler for reading NewsGroup with ArticleList from File
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.10.1 [25-02-2008]
 */
public class NewsGroupHandler extends FileContentHandler {
	protected NewsGroup m_group;
	protected String m_headerKey=null;
	protected Article m_article;
	
    private int cnt;
    public NewsGroupHandler() {
        super();
    }
    
    public NNTPMessage getMessage() {
        return m_group;
    }
    
    public void endDocument() throws SAXException {
    
    }

    public void startDocument() throws SAXException {
    }

    public void characters(char[] ch, int start, int length)
            throws SAXException {
       if (m_article!=null && m_headerKey!=null) {
          m_article.getHeader().addHeaderValue(m_headerKey,XmlHelper.decode(String.valueOf(ch,start,length)));
       }
    }

    public void endElement(String namespaceURI, String localName, String qName)
            throws SAXException {
    	
    	if (qName.equals("article")) {
    		m_group.addArticle(m_article);
            m_article=null;
        } 
        if (qName.equals(m_headerKey)) {
            m_headerKey=null;
        }

    }

    public void startElement(String namespaceURI, String localName,
            String qName, Attributes atts) throws SAXException {
        if (qName.equals("newsgroup")) {
            m_group=createNewsGroup(atts);
        } else if (qName.equals("article")) {
            m_article=createArticle(atts);
           	m_article.setHeader(new ArticleHeader());
            
            int size=m_group.getLast()-m_group.getFirst();
            cnt++;
            if (size< 100 || cnt%(int)(size*.01)==0) {
                advance(0,cnt,size);
            }
        } else if (!qName.equals("header") && !qName.equals("part")){
            m_headerKey=qName;
        } else if (qName.equals("part")) {
            m_article.addPart(Integer.parseInt(atts.getValue("seq"))-1,atts.getValue("id"));
        }

    }

    /*
     * TODO creating newsgroups from atts is done in two many places
     */
    private NewsGroupImpl createNewsGroup(Attributes atts) {
        NewsGroupImpl group = new NewsGroupImpl();
        group.setName(atts.getValue("name"));
        group.setFirst(Integer.parseInt(atts.getValue("first")));
        group.setLast(Integer.parseInt(atts.getValue("last")));
        group.setAllowPost("true".equals(atts.getValue("allowPost")));
        if (atts.getValue("lastUpdated")!=null)
        group.setLastUpdated(DateHelper.isoToDate(atts.getValue("lastUpdated")));
        return group;
    }

    private Article createArticle(Attributes atts) {
        Article a=new Article();
        a.setId(atts.getValue("id"));
        a.setNumber(Integer.parseInt(atts.getValue("number")));    
        a.setSaved(Boolean.parseBoolean(atts.getValue("saved")));
        if (atts.getValue("parts")!=null) {
            a.setTotalParts(Integer.parseInt(atts.getValue("parts")));
        }
        return a;
    }
    
}
