/* 	CfEditHeaderPane.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 15-jan-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 26, 2008	dennis@brains2b.nl		implemented CoffeeExchangeView.getSelectEvent()		
*/
package org.brains2b.coffee.gui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.KeyboardFocusManager;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.List;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.KeyStroke;
import javax.swing.text.BadLocationException;

import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.ArticleHeader;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.local.CoffeeMenu;
import org.brains2b.coffee.message.local.FileListMessage;
import org.brains2b.coffee.message.local.Identity;
import org.brains2b.coffee.plugin.CoffeeExchangeView;
import org.brains2b.swingext.action.DefaultAction;
import org.brains2b.swingext.gui.IconFactory;
import org.brains2b.swingext.gui.list.MutableListModel;
import org.brains2b.util.XmlHelper;

public class CfEditHeaderPane extends JPanel implements ActionListener, CoffeeExchangeView {

	class FileCellRenderer extends DefaultListCellRenderer {

		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			if (index==5) {
				value = "more files.. ";
			} else {
				value = ((File) value).getName();
			}
			return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
		}
		
	}
	class NewsGroupSelector implements ActionListener {
		
		public void actionPerformed(ActionEvent e) {
			String groupName = ((JMenuItem) e.getSource()).getText();
			m_newsgroups.replaceSelection((m_newsgroups.getSelectionStart()!=0?"\t":"")+groupName);
		}
	}
	
	class MouseHandler extends MouseAdapter {
		
		
		public void mousePressed(MouseEvent e) {
			if (e.isPopupTrigger()) {
				popup(e.getPoint());
			}
		}
		public void mouseReleased(MouseEvent e) {
			if (e.isPopupTrigger()) {
				popup(e.getPoint());
			}
		}
		
	}
	
	JPopupMenu m_pop= new JPopupMenu();
	NewsGroupSelector m_selector = new NewsGroupSelector();
	String m_part;
	Point m_lastPosition;
    private JComboBox m_from;
    private JTextField m_subject;
    private JTextPane m_newsgroups;
    private JList m_attach;
    private CfStatsDelegate m_stats;
    private ArticleHeader m_header;
    public CfEditHeaderPane() {
        super();
        init();
    }
    
    public void setHeader(ArticleHeader a) {
        m_header = a;
        if (a!=null) {
            if (a.getAuthor()!=null) {
                m_from.setSelectedItem(a.getAuthor());
            }
            m_newsgroups.setText(a.getHeaderValue("Newsgroups"));
            m_subject.setText(a.getSubject());
        }
    }
    
    public ArticleHeader getHeader() {
        if (m_header==null) m_header= new ArticleHeader();
        if (m_from.getSelectedItem() instanceof Identity) {
        	m_header.addHeaderValue("From", ((Identity) m_from.getSelectedItem()).getFrom());
        } else {
        	m_header.addHeaderValue("From", (String) m_from.getSelectedItem());
        }
        m_header.addHeaderValue("Newsgroups", m_newsgroups.getText());
        m_header.addHeaderValue("Subject", m_subject.getText());
        return m_header;
    }
    
    private void init() {
        setLayout(new GridBagLayout());
        GridBagConstraints gbc =new GridBagConstraints();
        gbc.anchor= GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(2,5,2,5);
        gbc.gridx=1;
        gbc.gridy=1;
        gbc.weightx=0d;
        add(new JLabel("From:"),gbc);
        m_from = new JComboBox();
        m_from.setEditable(true);
        createPreferredSize(m_from,20,1);
        gbc.weightx=0.50d;
        gbc.gridx++;
        add(m_from,gbc);
        gbc.gridx++;
        add(new JPanel(),gbc);
        gbc.gridx=1;
        gbc.gridy++;
        gbc.weightx=0d;
        add(new JLabel("Subject:"),gbc);
        m_subject = new JTextField();
        gbc.weightx=0.50d;
        gbc.gridwidth=2;
        gbc.gridx++;
        createPreferredSize(m_subject,60,1);
        add(m_subject,gbc);
        gbc.gridwidth=1;
        gbc.gridx=1;
        gbc.gridy++;
        gbc.weightx=0d;
        add(new JLabel("Newsgroup:"),gbc);
        gbc.gridx++;
        gbc.gridwidth=2;
        gbc.weightx=0.50d;
        m_newsgroups = new JTextPane();
        m_newsgroups.getActionMap().put("Newsgroup.suggest", new DefaultAction(this,"Newsgroup.suggest"));
        m_newsgroups.getInputMap(WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, KeyEvent.CTRL_DOWN_MASK), "Newsgroup.suggest");
        m_newsgroups.addMouseListener(new MouseHandler());
        m_newsgroups.setFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS,
        		m_subject.getFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS));
        m_newsgroups.setFocusTraversalKeys(KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS,
        		m_subject.getFocusTraversalKeys(KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS));
        createPreferredSize(m_newsgroups,60,2);
        m_newsgroups.setBorder(m_subject.getBorder());
        add(m_newsgroups,gbc);
        
        //Attach
        JButton btn = new JButton(IconFactory.createIcon("/org/brains2b/coffee/resource/Attach16.gif"));
        btn.setPreferredSize(new Dimension(24,24));
        btn.setActionCommand("Message.attach");
        btn.addActionListener(this);
        gbc.fill = GridBagConstraints.NONE;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.gridy++;
        gbc.gridx=1;
        gbc.gridwidth=1;
        add(btn,gbc);
        m_attach = new JList(new MutableListModel() {
        	public int getSize() {
        		int size = super.getSize();
        		if (size>6) size=6;
        		return size;
        	}
        	
        });
        m_attach.setLayoutOrientation(JList.HORIZONTAL_WRAP);
        m_attach.setBorder(m_subject.getBorder());
        m_attach.setVisibleRowCount(2);
        m_attach.setFixedCellWidth(180);
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridx++;
        gbc.gridwidth=2;
        m_attach.setCellRenderer(new FileCellRenderer());
        add(m_attach,gbc);
        gbc.gridy++;
        gbc.gridx=3;
        gbc.gridwidth=1;
        m_stats = new CfStatsDelegate(this);
        add(m_stats.addStats(Article.class),gbc);
    }
    
    private void createPreferredSize(JComponent c,int columns, int lines) {
    	FontMetrics fm = c.getFontMetrics(c.getFont());
    	c.setPreferredSize(new Dimension((int) (fm.getMaxAdvance()*columns/2),lines*fm.getHeight()+7));
    	c.setMinimumSize(new Dimension((int) (fm.getMaxAdvance()*columns/4),lines*fm.getHeight()+7));
    }

	public void actionPerformed(ActionEvent e) {
		if ("Newsgroup.suggest".equals(e.getActionCommand())) {
			popup(null);
		} else {
			ActionListener[] l = (ActionListener[]) listenerList.getListeners(ActionListener.class);
			for (int i=l.length-1;i>=0;i--) {
				l[i].actionPerformed(e);
			}
			
		}
		
	}
	
	private void popup(Point p) {
		int pos =m_newsgroups.getCaretPosition()-1;
		m_part = null;
		if (p==null) {
			try {
				p=m_newsgroups.modelToView(pos+1).getLocation();
			} catch (BadLocationException blex){}
		}
		m_lastPosition = p;
		String text = m_newsgroups.getText();
		if (pos>-1 && text.charAt(pos)=='.') {
			int first =0;
			for (int i=pos;i>=0;i--) {
				if (XmlHelper.isWhitespace(text.charAt(i))) {
					first=i;
					break;
				}
			}
			m_newsgroups.setSelectionStart(first);
			m_newsgroups.setSelectionEnd(pos+1);
			m_part = text.substring(first,pos+1).trim();
		} 
		TicketBroker.getBroker().broker(new CoffeeEvent(this,CoffeeEvent.GET_LIST,"subscription"));
	}
	
	public void setCurrentMessage(NNTPMessage message) {
		if (message instanceof CoffeeMenu) {
			if ("subscription".equals(message.getId())) {
				m_pop.removeAll();
				NNTPMessage[] list = ((CoffeeMenu) message).getArray();
				for (int i=0;i<list.length;i++) {
					if (m_part==null || (m_part!=null && list[i].getId().startsWith(m_part)) ) {
						JMenuItem item = new JMenuItem(list[i].getId());
						item.addActionListener(m_selector);
						m_pop.add(item);
					}
				}
				m_pop.show(m_newsgroups, m_lastPosition.x, m_lastPosition.y);
				m_pop.repaint();
			} else {
				NNTPMessage[] list = ((CoffeeMenu) message).getArray();
				for (int i=0;i<list.length;i++) {
					m_from.addItem(list[i]);
				}
				m_from.getEditor().selectAll();
			}
		} else if (message instanceof Article) {
			Article a = (Article) message;
			if (a.getFile()!=null) {
				List list =((FileListMessage) a.getFile()).getFiles();
				((MutableListModel) m_attach.getModel()).setData(list);
			} else {
				((MutableListModel) m_attach.getModel()).setData(null);
			}
			fireCoffeeEvent(m_stats,CoffeeEvent.CALCULATE_POSTS,a);
		}
	}
	
	public void addActionListener(ActionListener listener) {
		listenerList.add(ActionListener.class, listener);
	}
    
	public void removeActionListener(ActionListener listener) {
		listenerList.remove(ActionListener.class, listener);
	}
	
	public void addItemListener(ItemListener listener) {
		m_from.addItemListener(listener);
	}
	
	public void removeItemListener(ItemListener listener) {
		m_from.removeItemListener(listener);
	}

	public void activate() {
	}

	public void deactivate() {
		
	}

	public void fireCoffeeEvent(CoffeeExchange source, int action, Object argument) {
		TicketBroker.getBroker().broker(new CoffeeEvent(source,action,argument));
		
	}

	public JMenu[] getMenus() {
		return null;
	}
	
	public int getSelectEvent(int messageType) {
    	return CoffeeEvent.NONE;
	}
}
