/* 	CfServerPane.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 25-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 26, 2008	dennis@brains2b.nl	Use new NewsGroup interface, support DnD on group list, use getSelectEvent
 * 									Use new methods from Subscription
 * Apr 09, 2008	dennis@brains2b.nl	Unsubscribe was disabled for subscription 	
*/
package org.brains2b.coffee.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.ResourceBundle;

import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.gui.dnd.DragNewsGroup;
import org.brains2b.coffee.message.GroupList;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.NewsGroup;
import org.brains2b.coffee.message.NewsGroupImpl;
import org.brains2b.coffee.message.local.CoffeeMenu;
import org.brains2b.coffee.message.local.ServerDescriptor;
import org.brains2b.coffee.message.local.Subscription;
import org.brains2b.coffee.message.util.GroupComparator;
import org.brains2b.coffee.plugin.CoffeeExchangeView;
import org.brains2b.swingext.action.ExtActionMap;
import org.brains2b.swingext.gui.list.MutableListModel;

/**
 * [Short description]
 * [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version $Revision: 1.13 $ $Date: 2010/12/30 23:42:07 $
 */
public class CfServerPane extends JPanel implements CoffeeExchangeView,ActionListener {

    private class ActionHandler implements ActionListener, ListSelectionListener {
        
        public void actionPerformed(ActionEvent e) {
            if (m_loading) {
                return;
            }
        	if (e.getSource()==m_serverList) {
        	    handleServerChange((ServerDescriptor) m_serverList.getSelectedItem());
        	}
        }
        public void valueChanged(ListSelectionEvent e) {
            if (e.getSource() instanceof JList && !e.getValueIsAdjusting()) {
                JList list= (JList) e.getSource();
                if (list==m_subscriptionList) {
                    handleSubscriptionChange((Subscription) list.getSelectedValue());
                } else {
        	       handleGroupChange((NewsGroup) list.getSelectedValue());
                }
        	}

        }
    }
    
    private class MouseHandler extends MouseAdapter {
        public void mousePressed(MouseEvent e) {
            contextMenu(e);
            super.mousePressed(e);
        }
        
        public void mouseReleased(MouseEvent e) {
            contextMenu(e);
            super.mouseReleased(e);
        }
        
        private void contextMenu(MouseEvent e) {
            if (e.isPopupTrigger() || e.isMetaDown()) {
                JPopupMenu pop=new JPopupMenu();
                Object[] keys=getActionMap().allKeys();
                for (int i=0;i<keys.length;i++) {
                        pop.add(getActionMap().get(keys[i]));
                }
                JList list=(JList) e.getComponent();
                int idx=e.getY()/list.getFixedCellHeight();
                if (idx>=0 && e.getY()<list.getModel().getSize()*list.getFixedCellHeight()) {
                    list.setSelectedIndex(idx);
                    pop.show(e.getComponent(),e.getX(),e.getY());
                }
            }
        }
        
    }
    
    private class TabHandler implements ChangeListener {
     	public void stateChanged(ChangeEvent e) {
            JTabbedPane pane=(JTabbedPane) e.getSource();
            int idx=pane.getSelectedIndex();
            if (idx<2) {
            	m_filter[idx].clearFilter();
            }
            if (idx==0) {
               	fireCoffeeEvent(CfServerPane.this,CoffeeEvent.GET_LIST,"subscription");
        		getActionMap().get("NewsGroups.subscribe").setEnabled(false);
                getActionMap().get("NewsGroups.unsubscribe").setEnabled(true);
                getActionMap().get("NewsGroups.getlist").setEnabled(false);
                m_subscriptionList.clearSelection();
            } else {
                fireCoffeeEvent(CfServerPane.this,CoffeeEvent.GET_LIST,"server");
                if (m_groupList.getModel().getSize()==0) {
                    handleServerChange((ServerDescriptor) m_serverList.getSelectedItem());
                }
                getActionMap().get("NewsGroups.subscribe").setEnabled(true);
                getActionMap().get("NewsGroups.unsubscribe").setEnabled(false);
                getActionMap().get("NewsGroups.getlist").setEnabled(true);
                m_groupList.clearSelection();
            }
         	
           
        }   
    }
    private int[] m_selectEvent = new int[0];
    private JComboBox m_serverList;
    private JList m_groupList;
    private JList m_subscriptionList;
    //private JList m_folderList;
    private boolean m_loading;
    private CfFilterDelegate[] m_filter;
    private CfStatsDelegate[] m_stats;
    private CoffeeExchange m_parent;
    public CfServerPane(CoffeeExchange parent) {
     	super();
        m_parent =parent;
        init();
    }
    
    private void init() {
    	ResourceBundle rb = ResourceBundle.getBundle("org.brains2b.coffee.resource.coffee");
        m_filter=new CfFilterDelegate[2];
        m_stats=new CfStatsDelegate[2];
        
        m_filter[0]=new CfFilterDelegate(this);
        m_stats[0]=new CfStatsDelegate(this);
        m_filter[1]=new CfFilterDelegate(this);
        m_stats[1]=new CfStatsDelegate(this);
        
        ExtActionMap map=new ExtActionMap(this);
        map.addAction("NewsGroups.subscribe",rb.getString("NewsGroups.subscribe"),rb.getString("NewsGroups.subscribe.Tooltip"),null,"Ctrl Shift S",'s');
        map.addAction("NewsGroups.unsubscribe",rb.getString("NewsGroups.unsubscribe"),rb.getString("NewsGroups.unsubscribe.Tooltip"),null,"Ctrl Shift U",'u');
        map.addAction("NewsGroups.getlist",rb.getString("NewsGroups.getlist"),rb.getString("NewsGroups.getlist.Tooltip"),null,"Ctrl Shift L",'l');
        setActionMap(map);
        setLayout(new BorderLayout());
        ActionHandler ah=new ActionHandler();
        MouseHandler mh=new MouseHandler();
        
        JTabbedPane pane=new JTabbedPane();
        JPanel subPnl=new JPanel();
        subPnl.setLayout(new BorderLayout());
        m_subscriptionList=addList(subPnl,ah);
        m_subscriptionList.addMouseListener(mh);
        m_subscriptionList.setDragEnabled(true);
        DragNewsGroup dragNewsGroup = new DragNewsGroup();
        m_subscriptionList.setTransferHandler(dragNewsGroup);
        
        JPanel bottom=new JPanel();
        bottom.setLayout(new BorderLayout());
        bottom.add(m_stats[0].addStats(GroupList.class),BorderLayout.NORTH);
        bottom.add(m_filter[0].addFilter(7005,"newsGroups",10),BorderLayout.SOUTH);
        subPnl.add(bottom,BorderLayout.SOUTH);
        pane.addTab(rb.getString("ServerTab.Subscription"),null,subPnl,rb.getString("ServerTab.Subscription.Tooltip"));
        
        JPanel allPnl=new JPanel();
        allPnl.setLayout(new BorderLayout());
        m_serverList=new JComboBox();
        m_serverList.addActionListener(ah);
        allPnl.add(m_serverList,BorderLayout.NORTH);
        m_groupList=addList(allPnl,ah);
        m_groupList.addMouseListener(mh);
        m_groupList.setDragEnabled(true);
        m_groupList.setTransferHandler(dragNewsGroup);
        bottom=new JPanel();
        bottom.setLayout(new BorderLayout());
        bottom.add(m_stats[1].addStats(GroupList.class),BorderLayout.NORTH);
        bottom.add(m_filter[1].addFilter(8005,"newsGroups",10),BorderLayout.SOUTH);
        allPnl.add(bottom,BorderLayout.SOUTH);
        pane.addTab(rb.getString("ServerTab.All"),null,allPnl,rb.getString("ServerTab.All.Tooltip"));
        pane.addChangeListener(new TabHandler());
       
        add(pane,BorderLayout.CENTER);
        
        
    }
    public void fireCoffeeEvent(CoffeeExchange source, int action, Object argument) {
        TicketBroker.getBroker().broker(new CoffeeEvent(source,action,argument));
    }
    public void setCurrentMessage(NNTPMessage message) {
        m_loading=true;
    	if ("subscription".equals(message.getId())) {
            List list =((CoffeeMenu) message).getList();
            if (list==null) return;
            Collections.sort(list, GroupComparator.getInstance());
            (( MutableListModel) m_subscriptionList.getModel()).setData(list);
            m_filter[0].setCurrentMessage(message);
            m_stats[0].setCurrentMessage(message);
            getActionMap().get("NewsGroups.subscribe").setEnabled(false);
            getActionMap().get("NewsGroups.unsubscribe").setEnabled(true);
            getActionMap().get("NewsGroups.getlist").setEnabled(false);
        } else if (message instanceof GroupList) {
            List list= ((GroupList) message).getGroups();
            if (list==null) return;
            Collections.sort(list, GroupComparator.getInstance());
            if (list!=null) {
                m_groupList.setListData(list.toArray());
                m_groupList.repaint();
            }
            m_filter[1].setCurrentMessage(message);
            m_stats[1].setCurrentMessage(message);
            getActionMap().get("NewsGroups.subscribe").setEnabled(true);
            getActionMap().get("NewsGroups.unsubscribe").setEnabled(false);
            getActionMap().get("NewsGroups.getlist").setEnabled(true);
        } else if ("server".equals(message.getId())) {
            List lsd = ((CoffeeMenu) message).getList();
        	if (lsd.size()!=m_serverList.getItemCount()) {
	            m_serverList.removeAllItems();
	            for (Iterator it = lsd.iterator();it.hasNext();) {
   					m_serverList.addItem((ServerDescriptor) it.next());
	            }
            }
            
        }
    	m_loading=false;

    }
    private void handleServerChange(ServerDescriptor sd) {
    	m_filter[0].clearFilter();
        if (sd!=null) {
            fireCoffeeEvent(this, CoffeeEvent.CHANGE_SERVER, sd);
            fireCoffeeEvent(this, CoffeeEvent.XML_READ, sd);
        }
    }

     private void handleSubscriptionChange(Subscription sub) {
        if (sub!=null && findEvent(sub.getType())==CoffeeEvent.NEWS_GROUP) {
            fireCoffeeEvent(m_parent,CoffeeEvent.SUBSCRIPTION_GROUP,sub);
        }
    }
     
    private void handleGroupChange(NewsGroup group) {
        if (group!=null && group.getName()!=null && findEvent(group.getType())==CoffeeEvent.NEWS_GROUP) {
            fireCoffeeEvent(m_parent,CoffeeEvent.NEWS_GROUP,group.getName());
        }
    }
    
    private JList addList(JPanel pnl,ListSelectionListener ah) {
        JList list=new JList();
        list.addListSelectionListener(ah);
        JScrollPane scroll=new JScrollPane(list);
        NewsGroup prototype= new NewsGroupImpl();
        prototype.setName("alt.comp.java.lang");
        list.setModel(new MutableListModel());
        list.setPrototypeCellValue(prototype);
        list.setCellRenderer(new NewsGroupRenderer());
        pnl.add(scroll,BorderLayout.CENTER);
        return list;
    }
    
    public void actionPerformed(ActionEvent e) {
    	String actionCommand = e.getActionCommand();
    	if (actionCommand.equals("NewsGroups.subscribe")) {
           NewsGroup group=(NewsGroup) m_groupList.getSelectedValue();
           if (group!=null) {
                Subscription s = Subscription.createSubscription(group,(ServerDescriptor) m_serverList.getSelectedItem());
           		fireCoffeeEvent(this,CoffeeEvent.LIST_ADD,s);
           }
       } else if (actionCommand.equals("NewsGroups.unsubscribe")) {
           NewsGroup group=(NewsGroup) m_subscriptionList.getSelectedValue();
           int idx = m_subscriptionList.getSelectedIndex();
           ((MutableListModel) m_subscriptionList.getModel()).removeData(idx);
           if (group!=null) {
               fireCoffeeEvent(this,CoffeeEvent.LIST_REMOVE,group);
           }
           
       } else if (actionCommand.equals("NewsGroups.getlist")) {
    	   ServerDescriptor sd = (ServerDescriptor) m_serverList.getSelectedItem();
    	   fireCoffeeEvent(this, CoffeeEvent.GET_GROUPLIST, sd);
       }
    }
    
    public JMenu[] getMenus() {
    	ResourceBundle rb = ResourceBundle.getBundle("org.brains2b.coffee.resource.coffee");
        JMenu[] menu=new JMenu[1];
        menu[0]=new JMenu(rb.getString("NewsGroups.menu"));
        menu[0].setMnemonic('g');
        menu[0].add(getActionMap().get("NewsGroups.subscribe"));
        menu[0].add(getActionMap().get("NewsGroups.unsubscribe"));
        menu[0].addSeparator();
        menu[0].add(getActionMap().get("NewsGroups.getlist"));
        return menu;
    }
    
    /* (non-Javadoc)
     * @see org.brains2b.coffee.plugin.CoffeeExchangeView#activate()
     */
    public void activate() {
       fireCoffeeEvent(this, CoffeeEvent.GET_LIST, "server");
       fireCoffeeEvent(this, CoffeeEvent.GET_LIST, "subscription");
    }
    
    /* (non-Javadoc)
     * @see org.brains2b.coffee.plugin.CoffeeExchangeView#deactivate()
     */
    public void deactivate() {
       
    }

    public void setSelectEvent(int messageId, int eventId) {
    	for (int i=0;i<m_selectEvent.length;i+=2) {
			if (m_selectEvent[i]==messageId) {
				m_selectEvent[i+1]= eventId;
				return;
			}
			
		}
    	int[] events = new int[m_selectEvent.length+2];
		System.arraycopy(m_selectEvent,0,events,0,m_selectEvent.length);
		events[events.length-2] = messageId;
		events[events.length-1] = eventId;
		m_selectEvent = events;
    }
    
	public int getSelectEvent(int messageId) {
		return findEvent(messageId);
	}
	
	private int findEvent(int messageId) {
		for (int i=0;i<m_selectEvent.length;i+=2) {
			if (m_selectEvent[i]==messageId) return m_selectEvent[i+1];
		}
		return CoffeeEvent.NONE;
	}
}
