/* 	CfStatusBar.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 26-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * May 14, 2008	dennis@brains2b.nl		Show text message
 * 18-05-2008	dennis@brains2b.nl		Set min-max for progressbar just at start		
*/
package org.brains2b.coffee.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;

import org.brains2b.coffee.Ticket;
import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.message.InfoMessage;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.local.ErrorMessage;
import org.brains2b.coffee.plugin.CoffeeExchangeView;
import org.brains2b.swingext.dialog.DialogHelper;
import org.brains2b.task.TaskEvent;
import org.brains2b.task.TaskListener;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class CfStatusBar extends JPanel implements TaskListener,ActionListener,CoffeeExchangeView {
    
	class CleanMessage extends TimerTask {
		@Override
		public void run() {
			m_status.setText(null);
			
		}
		
	}
	
    private JProgressBar m_progressBar;
    private JLabel m_status;
	private JButton m_queue;
	private Ticket m_lastTicket;
	private Timer m_timer;
	private boolean m_timerCanceled;
	private CleanMessage m_cleaner;
    /**
     * 
     */
    public CfStatusBar() {
		init();
		m_timer = new Timer();
    }
    
    private void init() {
        ResourceBundle rb = ResourceBundle.getBundle("org.brains2b.coffee.resource.coffee");
		setBorder(new EmptyBorder(2,0,1,0));
		m_status=new JLabel();
		m_progressBar=new JProgressBar();
		m_progressBar.setPreferredSize(new Dimension(240,22));
        m_progressBar.setStringPainted(true);
        m_queue = new JButton("0 / 0");
        m_queue.setToolTipText(rb.getString("Queue.Tooltip"));
        m_queue.addActionListener(this);
        add(m_status);
        add(m_queue);
		add(m_progressBar);
		
    }
    public void taskProgressed(TaskEvent e) {
		if (e.getSource() instanceof TicketBroker) {
			m_queue.setText(e.getPosition() + " / " + e.getMax());
		} else if (e.getSource() instanceof Ticket) {
			
			Ticket t = (Ticket) e.getSource();
			
			if (m_lastTicket==null) {
				m_lastTicket = t;
			}
			
			if (m_lastTicket.getSequence()<t.getSequence()) {
				return;
			}
			
			m_lastTicket = t;
				
			if (e.isFinished()) {
				m_lastTicket = null;	
            	if (t.getMessage()!=null && t.getMessage().getType()<0) {
            		displayMessage((InfoMessage) t.getMessage());
            	}
        		m_progressBar.setValue(0);
            	setProgressIndeterminate(m_progressBar,false);
            	m_progressBar.setString("");
            	return;
			} else { 
				m_progressBar.setString(t.getDescription());
				if (e.getMax()==e.getPosition()) {
                   	setProgressIndeterminate(m_progressBar,true);
                } else if (e.getMax()>e.getPosition()) {
                    setProgressIndeterminate(m_progressBar,false);
    			}
    			
    			if (m_progressBar.getValue()==0 && e.getMax()!=m_progressBar.getMaximum()) {
                	m_progressBar.setMaximum(e.getMax());
                	m_progressBar.setMinimum(e.getMin());
                }
    			m_progressBar.setValue(e.getPosition());
			}
			
		}
		
		
	}
    
    private final void setProgressIndeterminate(final JProgressBar bar, final boolean b) {
    	if (bar.isIndeterminate()!=b) {
    		SwingUtilities.invokeLater(new Runnable() {
    			public void run() {
    				bar.setIndeterminate(b);
    			}
    		});
    	}
    }

	public void actionPerformed(ActionEvent e) {
		if (e.getSource()==m_queue) {
			CfQueView queView = new CfQueView();
			queView.setThreads(TicketBroker.getBroker().getThreads());
			queView.setQueue(TicketBroker.getBroker().getQueList());
			TicketBroker.getBroker().addTaskListener(queView);
			DialogHelper.showDialog(getParent(), queView, "Queue");
		}
	}
    
	private void displayMessage(InfoMessage message) {
		if (m_timerCanceled) return;
		m_status.setForeground(Color.black);
		if (message instanceof ErrorMessage) {
			if (m_cleaner!=null) m_cleaner.cancel(); 
			m_cleaner = new CleanMessage();
			m_timer.schedule(m_cleaner, 5000);
			m_status.setForeground(Color.red);
		}
		m_status.setText(message.getResponse());
	}

	public void activate() {
	}

	public void deactivate() {
		m_timer.cancel();
		m_timerCanceled = true;
		
	}

	public void fireCoffeeEvent(CoffeeExchange source, int action,
			Object argument) {
		
	}

	public JMenu[] getMenus() {
		return null;
	}

	public int getSelectEvent(int messageType) {
		return 0;
	}

	public void setCurrentMessage(NNTPMessage message) {
		
	}
    
}
