/* 	BinaryOptions.java
 *	Copyright (c) 2007-2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Sep 4, 2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 26, 2008	dennis@brains2b.nl		implemented CoffeeExchangeView.getSelectEvent()		
*/
package org.brains2b.coffee.gui.binary;

import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JSeparator;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.decode.DecoderFactory;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.gui.settings.CfOptionView;
import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.local.FileListMessage;
import org.brains2b.coffee.message.local.FileMessage;
import org.brains2b.coffee.message.local.Options;
import org.brains2b.coffee.util.SubjectHelper;
import org.brains2b.swingext.form.EasyForm;
import org.brains2b.swingext.form.FormAction;

public class BinaryOptions extends EasyForm implements CfOptionView {

	//TODO Apply and Undo should be removed when shown in Option Screen
	class OptionChangeHandler implements ItemListener, ChangeListener {

		public void stateChanged(ChangeEvent e) {
			if (!m_loading) {
				updateExample(m_article, m_file);
				fireStateChanged();
			}
		}

		public void itemStateChanged(ItemEvent e) {
			if (!m_loading) {
				updateExample(m_article, m_file);
				fireStateChanged();
			}
		}
		
	}
	
	private JComboBox m_encoding;
	private JSpinner  m_blocksize;
	
	private JCheckBox m_msgCount;
	private JCheckBox m_fileName;
	
	private JLabel m_example;
	private Article m_article;
	private FileMessage m_file;
	private Options m_options;
	private boolean m_loading;
	
	public BinaryOptions() {
		super(0,FormAction.BTN_APPLY+FormAction.BTN_UNDO);
		init();
	}
	
	public String getMessageId() {
		return "options/binary";
	}

	public String getShortTitle() {
		return "binary def.";
	}

	public void activate() {
		fireCoffeeEvent(this, CoffeeEvent.GET_LIST, "options/binary");

	}

	public Options getOptions() {
		Options options = new Options("binary");
		options.setBoolean("show-file-count", m_msgCount.isSelected());
		options.setBoolean("show-filename", m_fileName.isSelected());
		options.setString("encoding", (String) m_encoding.getSelectedItem());
		options.setString("block-size", m_blocksize.getValue().toString());
		return options;
	}
	
	public void deactivate() {
		boolean newz=false;
		if (m_options==null) {
			m_options = new Options("binary");
			newz=true;
		}
		m_options.setBoolean("show-file-count", m_msgCount.isSelected());
		m_options.setBoolean("show-filename", m_fileName.isSelected());
		m_options.setString("encoding", (String) m_encoding.getSelectedItem());
		m_options.setString("block-size", m_blocksize.getValue().toString());
		if (newz) fireCoffeeEvent(this, CoffeeEvent.LIST_ADD, m_options);
	}

	public void fireCoffeeEvent(CoffeeExchange source, int action,
			Object argument) {
		TicketBroker.getBroker().broker(new CoffeeEvent(source,action,argument));
	}

	public void setCurrentMessage(NNTPMessage message) {
		if (message instanceof Article) {
			m_article = (Article) message;
			updateExample(m_article,null);
			if (m_article.getFile()!=null && m_article.getFile().getOptions()!=null) {
				setCurrentMessage(m_article.getFile().getOptions());
			}
		} else if (message instanceof FileMessage) {
			updateExample(m_article, (FileMessage) message);
		} else if (message instanceof Options) {
			Options opt = (Options) message;
			m_encoding.setSelectedItem(opt.getString("encoding"));
			m_blocksize.setValue(Integer.decode(opt.getString("block-size")));
			m_msgCount.setSelected(opt.getBoolean("show-file-count"));
			m_fileName.setSelected(opt.getBoolean("show-filename"));
			m_options = opt;
		}

	}

	private void init() {
		OptionChangeHandler optionChange = new OptionChangeHandler();
		m_encoding = new JComboBox(DecoderFactory.getEncoders());
		m_encoding.addItemListener(optionChange);
		addField("Encode with", m_encoding, 1, 1, 2);
		m_blocksize = new JSpinner(new SpinnerNumberModel(100,10,450,10));
		m_blocksize.addChangeListener(optionChange);
		addField("Block-size",m_blocksize,1,2,1);
		addComponent(new JLabel("kb"), 3, 2);
		
		addComponent(new JSeparator(), 1, 3,3,1);
		m_msgCount = new JCheckBox("Show file count in subject");
		addComponent(m_msgCount,1,4,3,1);
		m_msgCount.addItemListener(optionChange);
		m_fileName = new JCheckBox("Show filename in subject");
		m_fileName.addItemListener(optionChange);
		addComponent(m_fileName,1,5,3,1);
		addComponent(new JSeparator(), 1, 6,3,1);
		m_example = new JLabel();
		m_example.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		addComponent(m_example, 1, 7,4,1);
		createPreferredSize(m_example, 40, 1);
		
		//application defaults
		m_loading=true;
		m_encoding.setSelectedIndex(0);
		m_blocksize.setValue(new Integer(210));
		m_msgCount.setSelected(true);
		m_fileName.setSelected(true);
		m_loading=false;
		
	}
	
	public void showFormButtons(boolean b) {
		for (int i=getComponentCount()-1;i>=0;i--) {
			if (getComponent(i) instanceof FormAction) {
				((FormAction) getComponent(i)).setVisible(b);
			}
		}
	}
	
	private void createPreferredSize(JComponent c,int columns, int lines) {
    	FontMetrics fm = c.getFontMetrics(c.getFont());
    	c.setPreferredSize(new Dimension((int) (fm.getMaxAdvance()*columns/2),lines*fm.getHeight()+7));
    	c.setMinimumSize(new Dimension((int) (fm.getMaxAdvance()*columns/4),lines*fm.getHeight()+7));
    }
	
	void updateExample(Article a, FileMessage fm) {
		if (a==null || a.getFile()==null) return;
		int idx=1;
		FileListMessage flm = (FileListMessage) a.getFile();
		int total = flm.getSize();
		if (fm!=null) 
		for (int i=0;i<total;i++) {
			if (fm.getFile().equals(flm.getFiles().get(i))) {
				idx = i+1;
				break;
			}
		}
		m_example.setText(SubjectHelper.createSubject(m_article, getOptions(), idx, total));
		
		
	}
	
	public void performApply() {
		deactivate();
	}
	
	public void performUndo() {
		setCurrentMessage(m_options);
	}

	public void addChangeListener(ChangeListener l) {
		listenerList.add(ChangeListener.class, l);
		
	}

	public void removeChangeListener(ChangeListener l) {
		listenerList.remove(ChangeListener.class, l);
	}

	public void fireStateChanged() {
		ChangeEvent e = new ChangeEvent(getOptions());
		ChangeListener[] l = (ChangeListener[]) listenerList.getListeners(ChangeListener.class);
		for (int i=0;i<l.length;i++) {
			l[i].stateChanged(e);
		}
	}
	
	public JMenu[] getMenus() {
		return null;
	}
	
	public Icon getIcon() {
		return null;
	}
	
	public int getSelectEvent(int messageType) {
    	return CoffeeEvent.NONE;
	}
}
