/* 	CfIdentity.java
 *	Copyright (c) 2006-2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 15-dec-2006
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 26, 2008	dennis@brains2b.nl		implemented CoffeeExchangeView.getSelectEvent()		
*/
package org.brains2b.coffee.gui.settings;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JTextField;
import javax.swing.event.ChangeListener;

import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.local.Identity;
import org.brains2b.swingext.dialog.EditorPane;
import org.brains2b.swingext.form.EasyForm;
import org.brains2b.swingext.gui.IconFactory;
import org.brains2b.swingext.gui.list.ExtMutableList;

public class CfIdentity extends EasyForm implements CfOptionView, ActionListener {

	private JTextField m_id;
	private JTextField m_from;
	private ExtMutableList m_tagLines;
	
	private JTextField m_stubField = new JTextField(); //orphan used for EditorPane
	
	private Identity m_identity;
	
	public CfIdentity() {
		super(UPDATE);
		init();
	}
	
	
	private void init() {
		m_id = new JTextField();
		m_id.setColumns(20);
		m_id.setName("Id");
		addField("Alias", m_id, 0, 1, 3);
		
		m_from = new JTextField();
		m_from.setColumns(32);
		m_from.setName("From");
		addField("From", m_from, 0, 2, 4);
		
		addComponent(new JLabel("Taglines"), 0, 3);
		m_tagLines = new ExtMutableList();
		m_tagLines.addActionListener(this);
		addComponent(m_tagLines,0, 4, 4, 4);
	}
	
	public void activate() {
	}

	public void deactivate() {
	}

	public void fireCoffeeEvent(CoffeeExchange source, int action,
			Object argument) {
		TicketBroker.getBroker().broker(new CoffeeEvent(source,action,argument));

	}

	public JMenu[] getMenus() {
		return null;
	}

	public void setCurrentMessage(NNTPMessage message) {
		if (message==null) {
			m_identity =null;
			m_id.setText(null);
			m_from.setText(null);
			m_tagLines.clearData();
			return;
		}
		m_identity = (Identity) message;
		m_id.setText(m_identity.getId());
		m_from.setText(m_identity.getFrom());
		m_tagLines.setData(m_identity.getTagLines());
	}

	public void performApply() {
		setMode(UPDATE);
		if (m_identity==null) {
			m_identity = new Identity();
			setMode(ADD);
		}
		m_identity.setId(m_id.getText());
		m_identity.setContent("from", m_from.getText());
		m_identity.setTagLines(m_tagLines.getData());
		if (getMode()==ADD) {
			fireCoffeeEvent(this, CoffeeEvent.LIST_ADD, m_identity);
		}
	}

	public void performUndo() {
		setCurrentMessage(m_identity);
	}
	
	public Icon getIcon() {
		return IconFactory.createIcon("/org/brains2b/coffee/resource/Identity16.gif");
	}


	public String getMessageId() {
		return "identity";
	}


	public String getShortTitle() {
		return "identity";
	}


	public void actionPerformed(ActionEvent e) {
		if ("ExtMutableList.Add".equals(e.getActionCommand())) {
			m_stubField.setText(null); 
			EditorPane.showDialog(this, m_stubField, "Add a tagline" );
			if (m_stubField.getText()!=null) {
				m_tagLines.addItem(m_stubField.getText());
			}
 		} else if ("ExtMutableList.Edit".equals(e.getActionCommand())) {
			int index = m_tagLines.getSelectedIndex();
 			m_stubField.setText((String) m_tagLines.getSelectedValue()); 
			EditorPane.showDialog(this, m_stubField, "Edit this tagline" );
			if (m_stubField.getText()!=null) {
				m_tagLines.replaceItem(m_stubField.getText(),index);
			}
 		} else if ("ExtMutableList.Remove".equals(e.getActionCommand())) {
			int index = m_tagLines.getSelectedIndex();
			m_tagLines.removeItem(index);
 		}
		
	}


	public void addChangeListener(ChangeListener l) {
		listenerList.add(ChangeListener.class, l);
		
	}


	public void removeChangeListener(ChangeListener l) {
		listenerList.remove(ChangeListener.class, l);
		
	}

	public int getSelectEvent(int messageType) {
    	return CoffeeEvent.NONE;
	}
}
