/* 	CfServer.java
 *	Copyright (c) 2006-2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 16-dec-2006
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 26, 2008	dennis@brains2b.nl		implemented CoffeeExchangeView.getSelectEvent()		
*/
package org.brains2b.coffee.gui.settings;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.file.FileConnector;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.local.ServerDescriptor;
import org.brains2b.swingext.form.Form;
import org.brains2b.swingext.form.FormAction;
import org.brains2b.swingext.gui.IconFactory;

public class CfServer extends JPanel implements Form, CfOptionView {

	class ActionHandler implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			JFileChooser jfc = new JFileChooser();
			jfc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			int result =jfc.showOpenDialog(CfServer.this);
			if (result==JFileChooser.APPROVE_OPTION) {
				if ("path".equals(((JComponent) e.getSource()).getName())) {
					m_path.setText(jfc.getSelectedFile().getAbsolutePath());
				} else {
					m_download.setText(jfc.getSelectedFile().getAbsolutePath());
				}
			}
			
		}
	}
	class ChangeHandler implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			if (e.getSource() == m_auth) {
				setEnabled(m_userName,m_auth.isSelected());
				setEnabled(m_passWord,m_auth.isSelected());
				if (!m_auth.isSelected()) {
					m_userName.setText(null);
					m_passWord.setText(null);
				}
			}
		}
	}
	
    private ServerDescriptor m_server;
    private JTextField m_address;
    private JTextField m_port;
    private JTextField m_path;
    private JTextField m_download;
    private JSpinner m_maxConnections;
    private JCheckBox m_binary;
    private JCheckBox m_auth;
    private JTextField m_userName;
    private JPasswordField m_passWord;
    
    public CfServer() {
        super();
        init();
    }
    
    public void performApply() {
    }

    public void performCancel() {
        m_server=null;
        close();
    }

    public void performOk() {
        boolean newz=false;
        if (m_server==null) {
            m_server=new ServerDescriptor();
            newz=true;
        }
        m_server.setAddress(m_address.getText());
        m_server.setPort(Integer.parseInt(m_port.getText()));
        m_server.setPath(m_path.getText());
        m_server.setDownload(m_download.getText());
        m_server.setBinary(m_binary.isSelected());
	    if (m_auth.isSelected()) {
	        m_server.setUsername(m_userName.getText());
        	m_server.setPassword(new String(m_passWord.getPassword()));
        }
        Integer i= (Integer) m_maxConnections.getValue();
        m_server.setMaxConnections(i.intValue());
        fireCoffeeEvent(this,newz?CoffeeEvent.LIST_ADD:CoffeeEvent.EXPIRE_SERVER,m_server);
        close();
    }

    public void performUndo() {
        setServer(m_server);
        
    }
    
    private void setServer(ServerDescriptor server) {
        m_server = server;
        if (server!=null) {
            m_address.setText(server.getAddress());
            m_port.setText(String.valueOf(server.getPort()));
            m_path.setText(server.getPath());
            m_download.setText(server.getDownload());
            m_binary.setSelected(server.isBinary());
            if (server.getUsername()!=null) {
            	m_auth.setSelected(true);
            	m_userName.setText(server.getUsername());
            	m_passWord.setText(server.getPassword());
            } else {
            	m_auth.setSelected(false);
            }
            m_maxConnections.setValue(new Integer(server.getMaxConnections()));
        } else {
        	m_address.setText(null);
            m_port.setText("119");
            m_path.setText(new File(FileConnector.getSystemPath(),"servers").getAbsolutePath());
            m_download.setText(new File(FileConnector.getSystemPath(),"download").getAbsolutePath());
            m_maxConnections.setValue(new Integer(1));
        }
    }
   
    
    private void init() {
        ActionHandler actionHandler = new ActionHandler();
    	setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        setLayout(new GridBagLayout());
        GridBagConstraints gbc=new GridBagConstraints();
        gbc.fill=GridBagConstraints.HORIZONTAL;
        gbc.insets=new Insets(3,3,3,3);
        add(new JLabel("newsserver address"),gbc);
        m_address =new JTextField();
        m_address.setColumns(20);
        gbc.gridx=1;
        gbc.gridy=0;
        gbc.gridwidth=3;
        add(m_address,gbc);
        m_port = new JTextField();
        gbc.gridx+=3;
        gbc.gridwidth=1;
        add(m_port,gbc);
        gbc.gridy++;
        gbc.gridx=0;
        gbc.gridwidth=1;
        add(new JLabel("Maximum connections"),gbc);
        gbc.gridx=1;
        gbc.gridwidth=1;
        m_maxConnections=new JSpinner(new SpinnerNumberModel(1,1,25,1));
        add(m_maxConnections,gbc);
        gbc.gridx=3;
        gbc.gridwidth=3;
        m_binary=new JCheckBox("Contains binaries");
        add(m_binary,gbc);
        gbc.gridy++;
        gbc.gridx=0;
        gbc.gridwidth=1;
        add(new JLabel("location of files"),gbc);
        m_path = new JTextField();
        m_path.setColumns(30);
        gbc.gridx=1;
        gbc.gridwidth=4;
        add(m_path,gbc);
        JButton brws = new JButton(IconFactory.createIcon("/org/brains2b/coffee/resource/Open16.gif"));
        brws.addActionListener(actionHandler);
        brws.setName("path");
        gbc.gridx+=4;
        gbc.gridwidth=1;
        gbc.fill=GridBagConstraints.NONE;
        add(brws,gbc);
        gbc.gridy++;
        gbc.gridx=0;
        gbc.gridwidth=1;
        gbc.fill=GridBagConstraints.HORIZONTAL;
        add(new JLabel("download files to"),gbc);
        m_download = new JTextField();
        m_download.setColumns(30);
        gbc.gridx=1;
        gbc.gridwidth=4;
        add(m_download,gbc);
        JButton brwsd = new JButton(IconFactory.createIcon("/org/brains2b/coffee/resource/Open16.gif"));
        brwsd.addActionListener(actionHandler);
        brwsd.setName("download");
        gbc.gridx+=4;
        gbc.gridwidth=1;
        gbc.fill=GridBagConstraints.NONE;
        add(brwsd,gbc);
        gbc.fill=GridBagConstraints.BOTH;
        gbc.gridwidth=2;
        gbc.gridx=0;
        gbc.gridy++;
       	m_auth = new JCheckBox("Server requires login");
       	m_auth.addChangeListener(new ChangeHandler());
       	add(m_auth,gbc);
       	gbc.gridwidth=1;
        gbc.gridx=1;
        gbc.gridy++;
       	add(new JLabel("Username"),gbc);
       	gbc.gridx++;
       	gbc.gridwidth=2;
       	m_userName = new JTextField();
       	add(m_userName,gbc);
       	gbc.gridx=1;
        gbc.gridy++;
        gbc.gridwidth=1;
       	add(new JLabel("Password"),gbc);
       	gbc.gridx++;
       	gbc.gridwidth=2;
       	m_passWord = new JPasswordField();
       	add(m_passWord,gbc);
       	setEnabled(m_userName,false);
       	setEnabled(m_passWord,false);
        FormAction fa=new FormAction(this,FormAction.BTN_OK_CANCEL+FormAction.BTN_UNDO);
        gbc.gridx=0;
        gbc.gridwidth=6;
        gbc.gridy++;
        gbc.fill=GridBagConstraints.HORIZONTAL;
        add(fa,gbc);
    }

    public void setCurrentMessage(NNTPMessage message) {
        setServer((ServerDescriptor) message);
        
    }

    public void activate() {
        
    }

    public void deactivate() {
        
    }

    public void fireCoffeeEvent(CoffeeExchange source, int action, Object argument) {
        TicketBroker.getBroker().broker(new CoffeeEvent(source,action,argument));
    }

    public JMenu[] getMenus() {
        return null;
    }
    
    private void setEnabled(JComponent cmp, boolean enable) {
    	cmp.setEnabled(enable);
    	cmp.setOpaque(enable);
    }
    
    protected void close() {
		Component parent = getRootPane().getParent();
		if (parent instanceof JFrame) {
			((JFrame) parent).dispose();
		} else if (parent instanceof JDialog) {
			((JDialog) parent).dispose();
		} else if (parent instanceof JInternalFrame) {
			((JInternalFrame) parent).dispose();
		} else {
			parent.setVisible(false);
		}
	}

	public Icon getIcon() {
		return IconFactory.createIcon("/org/brains2b/coffee/resource/Server16.gif");
	}

	public String getMessageId() {
		return "server";
	}

	public String getShortTitle() {
		return "server";
	}

	public void addChangeListener(ChangeListener l) {
		listenerList.add(ChangeListener.class, l);
		
	}

	public void removeChangeListener(ChangeListener l) {
		listenerList.remove(ChangeListener.class, l);
	}
	
	public int getSelectEvent(int messageType) {
    	return CoffeeEvent.NONE;
	}
}
