/* 	ArticleNode.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 2-jan-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.gui.table;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;

import org.brains2b.coffee.message.Article;

public class ArticleNode implements MutableTreeNode {
    
    ArticleNode m_parent;
    Article m_article;
    ArrayList m_list;
    String m_title;
    
    public ArticleNode(String title) {
    	m_title = title;	
    }
    
    public ArticleNode(Article article) {
        m_article=article;
    }

    public boolean add(Object node) {
        if (m_list==null) {
            m_list=new ArrayList();
        }
        m_list.add(node);
        ((ArticleNode)node).setParent(this);
        return true;
    }
    
    public void insert(MutableTreeNode child, int index) {
        if (m_list==null) {
            m_list=new ArrayList();
        }
        m_list.add(index, child);
        child.setParent(this);
    }

    public void remove(int index) {
       if (m_list!=null) {
           m_list.remove(index);
       }
    }

    public void remove(MutableTreeNode node) {
       if (m_list!=null) {
           m_list.remove(node);
       }
    }

    public void removeFromParent() {
       m_parent.remove(this);
    }

    public void setParent(MutableTreeNode newParent) {
       m_parent = (ArticleNode) newParent;

    }

    public void setUserObject(Object object) {
        m_article = (Article) object;
    }

    public Enumeration children() {
        if (m_list==null || m_list.isEmpty()) {
            return null;
        }
        return Collections.enumeration(m_list);
    }

    public boolean getAllowsChildren() {
        return true;
    }

    public TreeNode getChildAt(int childIndex) {
        return (ArticleNode) m_list.get(childIndex);
    }

    public int getChildCount() {
        if (m_list==null) {
            return 0;
        }
        return m_list.size();
    }

    public int getIndex(TreeNode node) {
        if (m_list==null) {
            return -1;
        }
        return m_list.indexOf(node);
    }

    public TreeNode getParent() {
        return m_parent;
    }

    public boolean isLeaf() {
        return getChildCount()==0; 
    }

    public Article getArticle() {
        return m_article;
    }

    public void setArticle(Article article) {
        m_article = article;
    }
    
    public String toString() {
        if (m_article==null) {
            return m_title!=null?m_title:"{empty}";
        }
        return m_article.getSubject();
    }

    public boolean addAll(Collection c) {
        for (Iterator it = c.iterator();it.hasNext();) {
            add(it.next());
        }
        return true;
    }

    public void clear() {
        if (m_list!=null) {
            m_list.clear();
        }
    }
    
    public List getArticles() {
    	return m_list;
    }

	public String getTitle() {
		return m_title;
	}

	public void setTitle(String title) {
		m_title = title;
	}
   
}
