/* 	ArticleTableCellRenderer.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 3-jan-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.gui.table;

import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;

import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.table.TableCellRenderer;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeModel;

import org.brains2b.coffee.message.Article;
import org.brains2b.swingext.gui.IconFactory;

public class ArticleTreeCellRenderer extends JTree implements TableCellRenderer {

    private class TreeCellRenderer extends DefaultTreeCellRenderer {
        
        public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
            
        	Article a =(((ArticleTreeTableModel) getModel()).getArticle(row));
            
        	String type = a==null?null:a.getHeader().getMimeType();
        	if (a!=null && !a.isAvailable()) {
        		setLeafIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/Remove16.gif"));
        		setOpenIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/Remove16.gif"));
        		setClosedIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/Remove16.gif"));
        	} else if (type==null || type.startsWith("text/")) {
        		if (leaf) {
        			if (a!=null && a.isSaved()) {
        				setLeafIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/Saved16.png"));
        			} else if (a!=null && a.getTotalParts()==(a.getParts()==null?0:a.getParts().size()+1)) {
        				setLeafIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/Binary16.gif"));
        			} else {
        				setLeafIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/Text16.gif"));
        			}
        		} else {
        			setOpenIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/Foldertext.gif"));
        			setClosedIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/Foldertext.gif"));
        		}
        	} else {
        		if (leaf) {
        			if (a!=null && a.isSaved()) {
        				setLeafIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/Saved16.png"));
        			} else if (a.getTotalParts()==(a.getParts()==null?1:a.getParts().size()+1)) {
        				setLeafIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/Binary16.gif"));
        			} else {
        				setLeafIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/BinaryBroken16.gif"));
        			}
        		} else {
        			setOpenIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/Folderbin.gif"));
        			setClosedIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/Folderbin.gif"));
        		}
        	}
        	if (a!=null && a.isNew()) {
                setFont(getFont().deriveFont(Font.BOLD));
            } else if (getFont()!=null){
                setFont(getFont().deriveFont(Font.PLAIN));
            }
        	if (a!=null && m_table instanceof ArticleTable && ((ArticleTable) m_table).isShowGroup()) {
        		setToolTipText(a.getNewsGroup());	
        	}
        	Component c = super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf,
                    row, hasFocus);
        	
            return c;
        }
    }
    
    protected int visibleRow; 
    protected JTable m_table;
    
    public ArticleTreeCellRenderer(TreeModel model, JTable table) {
        super(model);
        m_table =table;
        setCellRenderer(new TreeCellRenderer());
        setToggleClickCount(2);
        
    }
    
    public void setBounds(int x, int y, int width, int height) {
        super.setBounds(x, 0, width, m_table.getHeight());
    }
    
    
    public void paint(Graphics g) { 
              g.translate(0, -visibleRow * getRowHeight());          
              super.paint(g); 
          } 
    public Component getTableCellRendererComponent(JTable table,          
              Object value, 
              boolean isSelected, 
              boolean hasFocus, 
              int row, int column) {
                  visibleRow = row; 
                  return this; 
              }
    
}
