/* 	NewIndicator.java
 *	Copyright (c) 2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Nov 9, 2008
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.gui.widget;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;

import javax.swing.ImageIcon;

import org.brains2b.swingext.gui.IconFactory;

/**
 * Round icon showing the number of counts set by {@link #setCount(int)}
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version $Revision: 1.3 $ $Date: 2010/07/05 00:56:22 $
 */
public class NewIndicator extends ImageIcon{
		public final static int WIDTH=16;
		public final static int HEIGHT=16;
		private int m_count;
		private Image m_image;
		private Font m_font;
		
		public NewIndicator(int count) {
			m_count = count;
			ImageIcon icn = IconFactory.createIcon("/org/brains2b/coffee/resource/NewIndicator.png");
			m_image = icn.getImage();
			m_font =  Font.decode("Dailog-PLAIN-9");
			
		}
		
		/* (non-Javadoc)
		 * @see javax.swing.ImageIcon#getIconHeight()
		 */
		public int getIconHeight() {
				return HEIGHT;
		}

		/* (non-Javadoc)
		 * @see javax.swing.ImageIcon#getIconWidth()
		 */
		public int getIconWidth() {
			return WIDTH;
		}

		
		/* (non-Javadoc)
		 * @see javax.swing.ImageIcon#paintIcon(java.awt.Component, java.awt.Graphics, int, int)
		 */
		public synchronized void paintIcon(
			Component c,
			Graphics g,
			int x,
			int y) {
			
			String cnt = String.valueOf(m_count);
			if (m_count>99) {
				cnt="99";
			}
			Font f= g.getFont();
			g.setFont(m_font);
			g.drawImage(m_image,x,y,null);
			FontMetrics fm = g.getFontMetrics(m_font);
			final int width = fm.stringWidth(cnt);
			final int height = fm.getAscent();
			g.setColor(Color.white);
			g.drawString(cnt,x+(WIDTH-width)/2, y+height-1+(HEIGHT-height)/2);
			g.setFont(f);
			
			
		}

		public void setCount(int count) {
			m_count = count;
		}
}
