/* 	GroupList.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 17-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 25, 2008	dennis@brains2b.nl		Uses Newsgroup Interface		
*/
package org.brains2b.coffee.message;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import org.brains2b.saxo.SAXWriter;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version $Revision: 1.10 $ $Date: 2009/10/05 02:17:18 $
 */
public class GroupList implements NNTPMessage {
	private boolean m_headerSet;
	protected int m_type;
	protected ArrayList m_group;
    /**
     * 
     */
    public GroupList() {
        super();
        m_group=new ArrayList();
    }

    public void addGroup(NewsGroup group) {
        m_group.add(group);
    }
    
    public void removeGroup(NewsGroup group) {
        m_group.remove(group);
    }
    
    public void removeGroup(String groupName) {
        m_group.remove(getGroup(groupName));
    }
    
    public NewsGroup getGroup(String name) {
        for (int i=0;i<m_group.size();i++) {
            if (name.equals(((NewsGroup) m_group.get(i)).getName())) {
                return (NewsGroup) m_group.get(i);
            }
        }
        return null;
    }
    
    /**
     * @deprecated use getGroups
     * @return String[]
     */
    public String[] getGroupNames() {
        String[] names =new String[m_group.size()];
        for (int i=0;i<m_group.size();i++) {
           names[i]=((NewsGroup) m_group.get(i)).getName();
        }
        Arrays.sort(names);
        return names;
    }
    
    public List getGroups() {
        return m_group;
    }
    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#addLine(java.lang.String)
     */
    public void addLine(String s) {
        NewsGroupImpl group=new NewsGroupImpl();
        group.addLine(s);
        if (group.getName()!=null) {
        	m_group.add(group);
        }

    }

    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#addHeader(int, java.lang.String)
     */
    public void addHeader(int id, String s) {
        m_type=id;
        m_headerSet=true;
    }

    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#isHeaderSet()
     */
    public boolean isHeaderSet() {
        return m_headerSet;
    }

    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#endMessage()
     */
    public void endMessage() {
        
    }

    public void writeMessage(SAXWriter sw) throws SAXException {
        AttributesImpl atts=new AttributesImpl();
        atts.addAttribute(null,null,"size","CDATA",String.valueOf(m_group.size()));
        sw.startElement("newsgroups",atts);
        Iterator it=m_group.iterator();
        while (it.hasNext()) {
            NewsGroupImpl group=(NewsGroupImpl) it.next();
            group.writeList(sw);
        }
        sw.endElement("newsgroups");
    }
    public int getType() {
        return m_type;
    }
    
    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#hasContent()
     */
    public boolean hasContent() {
        
        return true;
    }
    
    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#getSize()
     */
    public int getSize() {
        return m_group.size();
    }
    
    public String getId() {
        return "groups";
    }
}
