/* 	MultiPartFilter.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 27-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 25, 2008	dennis@brains2b.nl		Uses Newsgroup Interface and delegate of Newsgroup,
 * 										Changes for SubjectHelper 		
*/
package org.brains2b.coffee.message;

import java.util.Date;
import java.util.List;

import org.brains2b.coffee.message.local.Subscription;
import org.brains2b.coffee.util.SubjectHelper;
import org.brains2b.coffee.util.SubjectHelper.SubjectInfo;
import org.brains2b.saxo.SAXWriter;
import org.xml.sax.SAXException;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class MultiPartGroup implements NewsGroup {
	
	private SubjectHelper m_helper;
	private NewsGroup m_delegate;
	
    public MultiPartGroup() {
        super();
        m_delegate = new NewsGroupImpl();
        m_helper = new SubjectHelper();
    }
    
     public MultiPartGroup(NewsGroup group) {
        super();
        while (group instanceof MultiPartGroup) { // Don't embed MultiPartGroup
        	group = ((MultiPartGroup) group).getDelegate();
        }
        m_delegate = group;
        m_helper = new SubjectHelper();
    }
     
    public void addArticle(Article a) {
        if (!a.isNew()) {
            m_delegate.addArticle(a);
            return;
        }
       final String subject=a.getHeader().getSubject();
       final SubjectInfo fsi=m_helper.getTruncated(subject);
       if (fsi.isYenc) {
           a.getHeader().addHeaderValue("Content-Type","x-application/yEnc");
       }
       if (fsi.subject==null) {
    	   m_delegate.addArticle(a);
    	   return;
       }
       boolean found=false;
       List articles = getArticleList();
       int len = articles==null?0:articles.size();
       for (int i=len-1;i>=0;i--) { 
           Article fa=(Article) articles.get(i);
           final String fns=fa.getHeader().getSubject();
           if (fns!=null && fns.equals(fsi.subject)) {
                if (fa.getPart()>fsi.part) {
                    fa.addPart(fa.getPart()-2,fa.getId());
                    fa.setPart(fsi.part);
                    fa.setHeader(a.getHeader());
                    fa.setSubject(fsi.subject);
                    fa.getHeader().addHeaderValue("Bytes", String.valueOf(fa.getSize()+a.getSize()));
                    fa.setId(a.getId());
                    fa.setNumber(a.getNumber());
                } else {
                    fa.addPart(fsi.part-2,a.getId());
                    fa.getHeader().addHeaderValue("Bytes", String.valueOf(fa.getSize()+a.getSize()));
                }
                found=true;
                break;
           }
       }
       if (!found) {
           a.setSubject(fsi.subject);
           a.setTotalParts(fsi.total);
           a.setPart(fsi.part);
           m_delegate.addArticle(a);
       }
    }

	public void addHeader(int id, String s) {
		m_delegate.addHeader(id, s);
	}

	public void addLine(String s) {
		if (m_delegate.getType()==224 || m_delegate.getType()==Subscription.SUBSCRIPTION) {
            Article a=new Article();
            a.addLine(s);
            addArticle(a);
        } else {
        	m_delegate.addLine(s);
        }
	}

	public void endMessage() {
		m_delegate.endMessage();
	}

	public List getArticleList() {
		return m_delegate.getArticleList();
	}

	public int getEstimate() {
		return m_delegate.getEstimate();
	}

	public int getFirst() {
		return m_delegate.getFirst();
	}

	public String getId() {
		return m_delegate.getId();
	}

	public int getLast() {
		return m_delegate.getLast();
	}

	public Date getLastUpdated() {
		return m_delegate.getLastUpdated();
	}

	public String getName() {
		return m_delegate.getName();
	}

	public int getSize() {
		return m_delegate.getSize();
	}

	public int getType() {
		return m_delegate.getType();
	}

	public boolean hasContent() {
		return m_delegate.hasContent();
	}

	public boolean isAllowPost() {
		return m_delegate.isAllowPost();
	}

	public boolean isHeaderSet() {
		return m_delegate.isHeaderSet();
	}

	public void removeArticle(Article a) {
		m_delegate.removeArticle(a);
	}

	public void setAllowPost(boolean allowPost) {
		m_delegate.setAllowPost(allowPost);
	}

	public void setArticleList(List articleList) {
		m_delegate.setArticleList(articleList);
	}

	public void setEstimate(int estimate) {
		m_delegate.setEstimate(estimate);
	}

	public void setFirst(int first) {
		m_delegate.setFirst(first);
	}

	public void setLast(int last) {
		m_delegate.setLast(last);
	}

	public void setLastUpdated(Date lastUpdated) {
		m_delegate.setLastUpdated(lastUpdated);
	}

	public void setName(String name) {
		m_delegate.setName(name);
	}

	public void writeMessage(SAXWriter sw) throws SAXException {
		m_delegate.writeMessage(sw);
	}
    
	public void writeList(SAXWriter sw) throws SAXException {
		m_delegate.writeList(sw);
	}

	public NewsGroup getDelegate() {
		return m_delegate;
	}
	
}
