/* 	MessageFilter.java
 *	Copyright (c) 2005-2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 27-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 26, 2008	dennis@brains2b.nl		Cleaned up and implemented toString() to be more descriptive
 * 14-05-2008	dennis@brains2b.nl		Fix for not finding first word correctly
 * 18-12-2008	dennis@brains2b.nl		contains should be case insensitive		
*/
package org.brains2b.coffee.message.filter;

import java.util.Date;

import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.util.CfDateHelper;
import org.brains2b.saxo.SAXWriter;
import org.brains2b.util.DateHelper;
import org.brains2b.util.XmlHelper;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/** Message holding a single Query statement 
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.89.1 [26-02-2008]
 */
public class MessageFilter implements NNTPMessage {
    public final static String OPERATIONS[] = {"equals","contains","lesser","greater","regex"};
	
	private int    m_operator;
	private boolean m_not;
	private boolean m_kill;
	private String m_key;
	private Object m_value;
	
    public MessageFilter() {
        super();
    }
    
    public MessageFilter(String key,String operator, Object value) {
        this(key,false, operator,value,false);
    }
    
    public MessageFilter(String key,boolean not, String operator, Object value, boolean kill) {
        m_key = key;
        m_value = value;
        m_not = not;
        m_kill = kill;
        setOperator(operator);
        setValue(value);
      
    }
    
    public MessageFilter(String key,String value) {
        this(key,false,"contains",value,false);
    }
    
   
    public boolean isValid(Object value) {
        
        switch (m_operator) {
        case 1:
        	return m_not ^ (value.toString().toLowerCase().indexOf(m_value.toString().toLowerCase())>-1);
        case 2:
        	return m_not ^ ((Comparable) m_value).compareTo(value==null?m_value:value)>0;
        case 3:
        	return m_not ^ ((Comparable) m_value).compareTo(value==null?m_value:value)<0;
        case 4:
        	return m_not ^ value.toString().matches((String) m_value);
        case 0:
       	default:
        	return m_not ^(m_value.equals(value));
        }
        
    }

	public String getKey() {
		return m_key;
	}

	public void setKey(String key) {
		m_key = key;
	}

	public boolean isKill() {
		return m_kill;
	}

	public void setKill(boolean kill) {
		m_kill = kill;
	}

	public boolean isNot() {
		return m_not;
	}

	public void setNot(boolean not) {
		m_not = not;
	}

	public int getOperator() {
		return m_operator;
	}

	public void setOperator(int operator) {
		m_operator = operator;
	}
	
	public void setOperator(String operator) {
		for (int i=0;i<OPERATIONS.length;i++) {
        	if (OPERATIONS[i].equalsIgnoreCase(operator)) {
        		m_operator=i;
        		break;
        	}
        }
	}

	public Object getValue() {
		return m_value;
	}

	public void setValue(Object value) {
		m_value = value;
	}
	public void setValue(String value) {
		if (m_key.equals("Date")) {
        	m_value=CfDateHelper.parse(value);
        } else if (m_key.equals("Lines") || m_key.equals("Bytes")) {
        	m_value=Integer.decode(value);
        } else {
        	m_value=value;
        }
	}

	public void addHeader(int id, String s) {
	}

	public void addLine(String s) {
	}

	public void endMessage() {
	}

	public String getId() {
		return null;
	}

	public int getSize() {
		return 0;
	}

	public int getType() {
		return 0;
	}

	public boolean hasContent() {
		return false;
	}

	public boolean isHeaderSet() {
		return false;
	}

	public void writeMessage(SAXWriter sw) throws SAXException {
		AttributesImpl qAtts=new AttributesImpl();
        qAtts.addAttribute(null,null,"field","CDATA",getKey());
        qAtts.addAttribute(null,null,"operation","CDATA",MessageFilter.OPERATIONS[getOperator()]);
        qAtts.addAttribute(null,null,"not","CDATA",String.valueOf(isNot()));
        qAtts.addAttribute(null,null,"kill","CDATA",String.valueOf(isKill()));
        String s = getValue().toString();
        if (getValue() instanceof Date) {
        	qAtts.addAttribute(null, null, "type", "CDATA", "date");
        	s  = DateHelper.isoFromDate((Date) getValue(), true);
        }
        sw.startElement("query",qAtts);
        sw.setContent(XmlHelper.encode(s));
        sw.endElement("query");
		
	}
    
	public String toString() {
		return m_key + " "+ OPERATIONS[m_operator] +" " + m_value;
	}
	
	@Override
	public int hashCode() {
		return toString().hashCode();
	}
	
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof MessageFilter) {
			return hashCode() == obj.hashCode();
		}
		return false;
	}
	public String toRegEx() {
		StringBuffer sb = new StringBuffer();
		if (m_not) sb.append("^(");
		switch (m_operator) {
        case 0:
		case 1:
        	sb.append(".*").append(m_value.toString().toLowerCase()).append(".*");
        	break;
        case 2:
        case 3:
        	return null;
        case 4:
        default:
        	sb.append(m_value.toString());
        }
		if (m_not) sb.append(")");
		return sb.toString();
	}
}


