package org.brains2b.coffee.message.local;

import java.util.HashMap;
import java.util.Iterator;

import org.brains2b.saxo.SAXWriter;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/**
 * Options is a {@link CoffeeBean} wrapper around HashMap to set and retrieve user
 * settable options for CoffeeBreak.
 * <p>Id determines which set of options is stored in this Map f.e. "binary" for the options
 * for sending binary files.</p>
 * 
 * @author <A HREF="MAILTO:dennis@brains2b.org">dennis@brains2b.org</A>
 * @version $Revision: 1.5 $ $Date: 2010/12/30 23:42:27 $
 */
public class Options extends LocalMessage implements CoffeeBean {

	public final static int OPTIONS=1005;
	
	private String m_id;
	private HashMap m_options;
	
	public Options() {
		this(null);
	}
	
	public Options(String id) {
		super();
		m_id = id;
		m_options = new HashMap();
	}
	
	public void setAttributes(Attributes atts) {
		m_id = atts.getValue("id");
	}

	public void setContent(String name, Attributes atts, String value) {
		if ("option".equals(name)) {
			m_options.put(atts.getValue("id"), value);
		}

	}
	
	public String getId() {
		return m_id;
	}

	public void setId(String id) {
		m_id = id;
	}
	public int getSize() {
		return m_options.size();
	}

	public int getType() {
		return OPTIONS;
	}

	public boolean hasContent() {
		return !m_options.isEmpty();
	}

	public void writeMessage(SAXWriter sw) throws SAXException {
		AttributesImpl atts = new AttributesImpl();
		atts.addAttribute(null, "id", "id", "CDATA", m_id);
		sw.startElement("options", atts);
		for (Iterator it= m_options.keySet().iterator();it.hasNext();) {
			final String key = (String) it.next();
			//We are reusing the attributes impl, while the startElement
			//has already written the value to the Writer
			atts.setAttribute(0, null, "id", "id", "CDATA", key);
			sw.startElement("option", atts);
			sw.setContent((String) m_options.get(key));
			sw.endElement("option");
		}
		sw.endElement("options");

	}

	public boolean getBoolean(String key) {
		String value = (String) m_options.get(key);
		if (value==null|| !value.equals("true")) {
			return false;			
		}
		return true;
	}
	
	public void setBoolean(String key, boolean b) {
		m_options.put(key,String.valueOf(b));
	}
	
	public int getInt(String key) {
		String value = (String) m_options.get(key);
		if (value!=null) {
			return Integer.valueOf(value);			
		}
		return 0;
	}
	
	public void setInt(String key, int num) {
		m_options.put(key,String.valueOf(num));
	}
	
	public String getString(String key) {
		return (String) m_options.get(key);
	}
	
	public void setString(String key, String value) {
		m_options.put(key,value);
	}
}
