/* 	ServerDescriptor.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 12-aug-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.message.local;

import org.brains2b.saxo.SAXWriter;
import org.brains2b.util.StringHelper;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

public class ServerDescriptor extends LocalMessage implements CoffeeBean {

	public final static int SERVER=1001;
	
    private String m_address;
    private int m_port;
    private String m_path;
    private String m_download;
    private boolean m_binary;
    private int maxConnections;
    private String m_username;
    //TODO Password should be stored encrypted
    private String m_password;
    
    public ServerDescriptor() {
        super();
    }
    
    public ServerDescriptor(String address, int port, String path) {
        super();
        m_address=address;
        m_port=port;
        m_path=path;
    }
    
    public int getType() {
        return SERVER;
    }

    public void writeMessage(SAXWriter sw) throws SAXException {
        writeList(sw);
    }
    
    public void writeList(SAXWriter sw) throws SAXException {
        sw.startElement("server",getAttributes());
        sw.startElement("path", null);
        sw.setContent(getPath());
        sw.endElement("path");
        sw.startElement("download", null);
        sw.setContent(getDownload());
        sw.endElement("download");
        if (!StringHelper.isEmpty(getUsername())) {
	        sw.startElement("username", null);
	        sw.setContent(getUsername());
	        sw.endElement("username");
	        sw.startElement("password", null);
	        sw.setContent(getPassword());
	        sw.endElement("password");
        }
        sw.endElement("server");
        
    }

    public String getAddress() {
        return m_address;
    }

    public void setAddress(String address) {
        m_address = address;
    }

    public String getPath() {
        return m_path;
    }

    public void setPath(String path) {
        m_path = path;
    }

    public int getPort() {
        return m_port;
    }

    public void setPort(int port) {
        m_port = port;
    }

    public String getId() {
        return m_address;
    }

    public boolean isBinary() {
        return m_binary;
    }

    public void setBinary(boolean binary) {
        m_binary = binary;
    }

    public String getDownload() {
        return m_download;
    }

    public void setDownload(String download) {
        m_download = download;
    }

    public int getMaxConnections() {
        return maxConnections;
    }

    public void setMaxConnections(int maxConnections) {
        this.maxConnections = maxConnections;
    }
    
    public String toString() {
        return getId();
    }

    private Attributes getAttributes() {
        AttributesImpl atts=new AttributesImpl();
        atts.addAttribute(null, null, "address", "CDATA", getAddress());
        atts.addAttribute(null, null, "port", "CDATA", String.valueOf(getPort()));
        atts.addAttribute(null, null, "binary", "CDATA", String.valueOf(isBinary()));
        atts.addAttribute(null, null, "maxConnections", "CDATA", String.valueOf(getMaxConnections()));
        return atts;
    }
    public void setAttributes(Attributes atts) {
        setAddress(atts.getValue("address"));
        setPort(119);
        if (atts.getValue("port")!=null) {
            try {
                setPort(Integer.parseInt(atts.getValue("port")));
            } catch (NumberFormatException nfex) {
            }
        }
        setBinary(new Boolean(atts.getValue("binary")).booleanValue());
        setMaxConnections(1);
        try {
            setMaxConnections(Integer.parseInt(atts.getValue("maxConnections")));
        } catch (NumberFormatException nfex) {
            
        }
    }

    public void setContent(String name, Attributes atts, String value) {
       if (name.equals("path")) {
           setPath(value);
       } else if (name.equals("download")) {
           setDownload(value);
       } else if (name.equals("username")) {
    	   setUsername(value);
       } else if (name.equals("password")) {
    	   setPassword(value);
       }
        
    }

	public String getPassword() {
		return m_password;
	}

	public void setPassword(String password) {
		m_password = password;
	}

	public String getUsername() {
		return m_username;
	}

	public void setUsername(String username) {
		m_username = username;
	}
	public boolean equals(Object obj) {
		if (obj instanceof ServerDescriptor) {
			ServerDescriptor sd= (ServerDescriptor) obj;
			return (sd.getAddress().equals(getAddress()) && getPort()==getPort()); 
		}
		return false;
	}
	
	public boolean hasContent() {
        return true;
    }

    public int getSize() {
        return 1;
    }
}
