/* 	Subscription.java
 *	Copyright (c) 2005-2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 27-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 25, 2008	dennis@brains2b.nl		Changes for easy use in NewsSearch		
*/
package org.brains2b.coffee.message.local;

import org.brains2b.coffee.message.NewsGroup;
import org.brains2b.coffee.message.NewsGroupImpl;
import org.brains2b.saxo.SAXWriter;
import org.brains2b.util.DateHelper;
import org.brains2b.util.XmlHelper;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/** Subscription is an implementation of NewsGroup holding
 * the groups a user is actually subscribed to.
 * <p>Holds server as extra information to make sure it can
 * be accessed directly</p>
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.90 [25-02-2008]
 */
public class Subscription extends NewsGroupImpl implements CoffeeBean, Cloneable {
	public final static int SUBSCRIPTION=1003;
	
	private String m_server;
    
	public Subscription() {
		super();
	}
	
    public int getType() {
        return SUBSCRIPTION;
    }
    
    /**
     * get the name of the server this subscription resides on
     * @return String, the name of the server
     */
    public String getServer() {
        return m_server;
    }
    
    /**
     * set the name of the server this subscription resides on
     * @param server String, the name of the server
     */
    public void setServer(String server) {
        m_server = server;
    }
    
    public void writeMessage(SAXWriter sw) throws SAXException {
        sw.insertElement("subscription",getAttributes());
    }
    
    protected Attributes getAttributes() {
        AttributesImpl atts=new AttributesImpl(super.getAttributes());
        atts.addAttribute("","","server","CDATA",m_server);
        return atts;
    }

    public void setAttributes(Attributes atts) {
        if (atts.getLength()==0) return;
        setServer(atts.getValue("server"));
        setName(XmlHelper.decode(atts.getValue("name")));
        setFirst(Integer.parseInt(atts.getValue("first")));
        setLast(Integer.parseInt(atts.getValue("last")));
        setAllowPost("true".equals(atts.getValue("allowPost")));
        setLastUpdated(DateHelper.isoToDate(atts.getValue("lastUpdated")));
    }

    public void setContent(String name, Attributes atts, String value) {
        
    }
    
    public final static Subscription createSubscription(NewsGroup group, ServerDescriptor server) {
    	if (group instanceof Subscription) return (Subscription) group; 
    	Subscription s =new Subscription();
        s.setAllowPost(group.isAllowPost());
        s.setFirst(group.getFirst());
        s.setLast(group.getLast());
        s.setLastUpdated(group.getLastUpdated());
        s.setName(group.getName());
        s.setServer(server.getId());
        return s;
    }
    
    public Object clone() {
    	Subscription s =new Subscription();
        s.setAllowPost(isAllowPost());
        s.setFirst(getFirst());
        s.setLast(getLast());
        s.setLastUpdated(getLastUpdated());
        s.setName(getName());
        s.setServer(getServer());
        return s;
    }
}
