/* 	NZBDocument.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 22-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 25, 2008	dennis@brains2b.nl		Cleaned up, Use String.valueOf instead of new String()
 * TODO Implement writes for creating adhoc nzb documents		
*/
package org.brains2b.coffee.plugin.nzb;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.brains2b.coffee.file.FileContentHandler;
import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.local.FileMessage;
import org.brains2b.saxo.SAXWriter;
import org.brains2b.util.XmlHelper;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/** ContentHandler for reading NZB documents from Articles and File
 * <p>Also acts as a container for NzbFile elements 
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.89.1 [25-02-2008]
 */
public class NzbDocument extends FileContentHandler implements NzbContent,NNTPMessage {
	
    private String m_subject;
    private List<NzbFile> m_files;
    private NzbArticle m_article;
    private String m_lastName;
    private boolean m_failed = false;
	private boolean m_paused;
    
    /**
     * Constructor
     */
    public NzbDocument() {
        super();
    }

    public void endDocument() throws SAXException {
    }

    public void startDocument() throws SAXException {
        m_files=new ArrayList();

    }

    public void characters(char[] ch, int start, int length)
            throws SAXException {
        if (m_lastName.equals("group")) {
            NzbFile f=(NzbFile) m_files.get(m_files.size()-1);
            f.addGroup(String.valueOf(ch,start,length));
        } else if (m_lastName.equals("segment")) {
            m_article.setId(XmlHelper.decode(String.valueOf(ch,start,length)));
        }

    }

    public void endElement(String namespaceURI, String localName, String qName)
            throws SAXException {
        
    }

    public void startElement(String namespaceURI, String localName,
            String qName, Attributes atts) throws SAXException {
        if ("file".equals(qName)) {
            m_files.add(createFile(atts));
        } else if ("segment".equals(qName)) {
            NzbFile f=(NzbFile) m_files.get(m_files.size()-1);
            m_article=createArticle(atts);
        	f.addArticle(m_article);
        }
        m_lastName=qName;

    }
    
    public void addFile(NzbFile file) {
        if (m_files==null) {
            m_files =new ArrayList();
        }
        m_files.add(file);
    }
    
    public NzbFile[] getFiles() {
        NzbFile[] files=new NzbFile[m_files.size()];
        return (NzbFile[]) m_files.toArray(files);
    }
    
    private NzbFile createFile(Attributes atts) {
        NzbFile f=new NzbFile();
        f.setSubject(XmlHelper.decode(atts.getValue("subject")));
        f.setPoster(XmlHelper.decode(atts.getValue("poster")));
        if (atts.getValue("date")!=null)
        f.setDate(new Date(Long.parseLong(atts.getValue("date"))*1000));
        return f;
    }
    
    private NzbArticle createArticle(Attributes atts) {
        NzbArticle a=new NzbArticle();
        a.setSequence(Integer.parseInt(atts.getValue("number")));
        a.setSize(Integer.parseInt(atts.getValue("bytes")));
        
        return a;
    }

    public NNTPMessage getMessage() {
        return new FileMessage();
    }
    
    public void setSubject(String name) {
        m_subject = name;
    }
    
    public NzbArticle findArticle(Article a) {
        NzbFile[] files=getFiles();
        for (int i=0;i<files.length;i++) {
            int idx=files[i].getArticleList().indexOf(a);
            if (idx>-1) {
                return (NzbArticle) files[i].getArticleList().get(idx);
            }
        }
        return null;
    }
    
    public NzbFile findFile(Article a) {
        NzbFile[] files=getFiles();
        for (int i=0;i<files.length;i++) {
            if (files[i].getArticleList().contains(a)) {
                return files[i];
            }
            
        }
        return null;
    }
    
    public int getType() {
        return 1;
    }
    
    public String getSubject() {
        return m_subject;
    }
          
    public boolean isComplete() {
        boolean result=true;
        NzbContent[] articles=getFiles();
        for (int i=0;i<articles.length;i++) {
            if (!articles[i].isComplete()) {
                result=false;
                break;
            }
        }
        return result;
    }

    public int getSize() {
        if (m_files==null) return 0;
        return m_files.size();
    }
    
    public boolean isFailed() {
        return m_failed;
    }

    public void setFailed(boolean failed) {
        m_failed = failed;
    }

	public void addHeader(int id, String s) {
	}

	public void addLine(String s) {
	}

	public void endMessage() {
	}

	public String getId() {
		return null;
	}

	public boolean hasContent() {
		return m_files!=null && m_files.isEmpty();
	}

	public boolean isHeaderSet() {
		return false;
	}

	public void writeMessage(SAXWriter sw) throws SAXException {
		sw.insertDescriptor();
		AttributesImpl atts= new AttributesImpl();
		atts.addAttribute(null, null, "xmlns", "CDATA", "http://www.newzbin.com/DTD/2003/nzb");
		sw.startElement("nzb",atts);
		
		for (NzbFile f:m_files) {
			f.writeMessage(sw);
		}
		
		sw.endElement("nzb");
	}

	public boolean isPaused() {
		return m_paused;
	}

	public void setPaused(boolean b) {
		m_paused = b;
	}
}
