/* 	NzbFile.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 22-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.plugin.nzb;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.brains2b.saxo.SAXWriter;
import org.brains2b.util.XmlHelper;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class NzbFile implements NzbContent {
	
	private String m_poster;
	private Date m_date; 
    private ArrayList<String> m_groups;
    private ArrayList<NzbArticle> m_articles;
    private String m_subject;
    private boolean m_failed=false;
	private boolean m_paused;
    /**
     * 
     */
    public NzbFile() {
        super();
        m_groups=new ArrayList();
        m_articles=new ArrayList();
    }

    public void addGroup(String name) {
        m_groups.add(name);
    }
    
    public void addArticle(NzbArticle article) {
     	m_articles.add(article);   
    }
    
    public String[] getGroups() {
        String[] groups=new String[m_groups.size()];
        return (String[]) m_groups.toArray(groups);
    }
    
    public List getArticleList() {
        return m_articles;
    }
    
    public NzbArticle[] getArticles() {
        NzbArticle[] articles=new NzbArticle[m_articles.size()];
        return (NzbArticle[]) m_articles.toArray(articles);
    }
    public String getSubject() {
        return m_subject;
    }
    public void setSubject(String name) {
        m_subject = name;
    }
    
    public boolean isComplete() {
        boolean result=true;
        NzbArticle[] articles=getArticles();
        for (int i=0;i<articles.length;i++) {
            if (!articles[i].isComplete()) {
                result=false;
                break;
            }
        }
        return result;
    }
    
    public int getSize() {
        if (m_articles!=null) {
        	return m_articles.size();
        } else {
            return 0;
        }
    }
    
    public boolean isFailed() {
        return m_failed;
    }
    public void setFailed(boolean failed) {
        m_failed = failed;
    }
    /* (non-Javadoc)
     * @see org.brains2b.coffee.plugin.nzb.NzbContent#getType()
     */
    public int getType() {
        return 2;
    }

	public boolean isPaused() {
		return m_paused;
	}

	public void setPaused(boolean b) {
		m_paused = b;
	}

	public String getPoster() {
		return m_poster;
	}

	public void setPoster(String poster) {
		m_poster = poster;
	}

	public Date getDate() {
		return m_date;
	}

	public void setDate(Date date) {
		m_date = date;
	}
	
	public void writeMessage(SAXWriter sw) throws SAXException {
		AttributesImpl atts = new AttributesImpl();
		atts.addAttribute(null, null, "subject", "CDATA", XmlHelper.encode(getSubject()));
		atts.addAttribute(null, null, "poster", "CDATA", getPoster());
		if (getDate()!=null)
		atts.addAttribute(null, null, "date", "CDATA", String.valueOf(getDate().getTime()/1000));
		sw.startElement("file", atts);
		
		if (m_groups!=null && !m_groups.isEmpty()) {
    		sw.startElement("groups", null);
			for (String g:m_groups) {
    			sw.startElement("group",null);
    			sw.setContent(g);
    			sw.endElement("group");
    		}
			sw.endElement("groups");
		}
		
		if (m_articles!=null && !m_articles.isEmpty()) {
			sw.startElement("segments", null);
    		for (NzbArticle a:m_articles) {
    			a.writeList(sw);
    		}
    		sw.endElement("segments");
		}
		sw.endElement("file");	
	}
}
