/* 	NZBProcessor.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 17-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]				[What]
 * Feb 24, 2008	dennis@brains2b.nl	Deserialized collections			
*/
package org.brains2b.coffee.plugin.nzb;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.brains2b.coffee.Ticket;
import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.plugin.CoffeeExchangeControl;
import org.brains2b.coffee.util.Yenc;
import org.brains2b.log.Logger;
import org.brains2b.task.DefaultTask;
import org.brains2b.task.TaskEvent;
import org.brains2b.task.TaskListener;
import org.brains2b.util.StringHelper;

/** Processor for handling NZB-files by sending out individual article
 * retrieval messages and collection and organizing them here
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.89.1 [24-02-2008]
 */
public class NzbProcessor extends DefaultTask implements CoffeeExchangeControl, TaskListener{
	NzbDocument m_doc;
	File m_path;
	
	ArrayList m_listeners;
	HashMap m_decoders;
    
	/**
     * Constructor
     * @param doc NzbDocument, the document to process
     * @param path File, the path to save the files downloaded from
     * the document 
     */
    public NzbProcessor(NzbDocument doc, File path) {
        m_doc=doc;
        m_path=path;
        m_decoders=new HashMap();
    }

    public void run() {
        setFinished(false);
        NzbFile[] files=m_doc.getFiles();
        for (int i=0;i<files.length;i++) {
           try {
           		if (!checkFile(files[i])) {
                    getFile(files[i]);
                } else {
                    fireTaskEvent(new TaskEvent(files[i],0,0,files[i].getSize(),files[i].getSize()));
                }
           } catch (Exception ex) {
           		Logger
           		.printException(ex);
           }
        }
        setFinished(true);
    }
    
    private boolean checkFile(NzbFile file) {
        String name=file.getSubject();
        int idx=name.indexOf("\"");
        if (idx>0 ) {
            int last=StringHelper.matchBracket(idx,name);
            if (last>idx || last==-1) {
            	if (last==-1) {last=idx;idx=-1;}
                name=name.substring(idx+1,last);
                File f=new File(m_path,name);
                if (f.exists() ) { //&& f.length()==file.getSize()) {
                    NzbArticle[] as=file.getArticles();
                    for (int i=0;i<as.length;i++) {
                        as[i].setComplete(true);
                    }
                    return true;
                } else {
                    return false;
                }
            }
        }
        return false;
        
    }
    
   	private void getFile(NzbFile file) throws Exception {
   	   	NzbArticle[] articles=file.getArticles();
   	   	for (int i=0;i<articles.length;i++) {
   	   	    articles[i].setNewsGroup(file.getGroups()[0]);
   	   	    articles[i].setSubject(file.getSubject());
   	   	    fireCoffeeEvent(this,CoffeeEvent.GET_ARTICLE,articles[i]);
   	    }
   	   	
   	}
    
   	private Yenc getDecoder(NzbFile file) {
   	    Yenc yenc=(Yenc) m_decoders.get(file);
   	    if (yenc==null) {
   	        yenc=new Yenc();
   	        m_decoders.put(file,yenc);
   	    }
   	    return yenc;
   	}
   	
   	private void removeDecoder(NzbFile file) {
   	    if (m_decoders.containsKey(file)) {
   	        m_decoders.remove(file);
   	    }
   	}
   	
   	public void taskProgressed(TaskEvent e) {
       	if (e.isFinished()) {
   	    	Ticket t=(Ticket) e.getSource();
            boolean failed=false;
            Article a=(Article) t.getEvent().getArgument();
            if (t.getMessage() instanceof Article) {
   	    	   a=(Article) t.getMessage();
            } else {
                failed=true;
            }
            
    	    NzbArticle na=m_doc.findArticle(a);
    	    NzbFile file=m_doc.findFile(a);
   	    	try {
                na.setComplete(true);
   	    		if (file!=null && !failed) {
   	    		    Yenc yenc=getDecoder(file);
   	    	    	yenc.decodePiece(a.getBody().getContent());
   	    	    	if (file.isComplete()) {
   	    	    	    yenc.decode(m_path);
   	    	    	    removeDecoder(file);
   	    	    	}
   	    		}
                na.setFailed(failed);
                file.setFailed(file.isFailed() || failed);    
   	    	} catch (Exception ex) {
   	    	    na.setFailed(true);
   	    	    file.setFailed(true);
   	    	    Logger.printException(ex);
   	    	} finally {
                if (!failed && a!=null) {
                    a.setBody(null);
                }   
            }
       	}   
   	   fireTaskEvent(e);
       	
       	
    }

	public void fireTaskEvent(TaskEvent e) {
	    if (m_listeners!=null) {
	        Iterator it=m_listeners.iterator();
	        while (it.hasNext()) {
	            ((TaskListener) it.next()).taskProgressed(e); 
	        }
	    }
	}

	public void addTaskListener(TaskListener tl) {
		if (m_listeners==null) {
			m_listeners=new ArrayList();
		}
		m_listeners.add(tl);
	}
	
	public void removeTaskListener(TaskListener tl) {
		if (m_listeners!=null) {
		    m_listeners.remove(tl);
		}
	}
	
	public TaskListener getTaskListener() {
		return this;
	}
    
    public void fireCoffeeEvent(CoffeeExchange source, int action, Object argument) {
        TicketBroker.getBroker().broker(new CoffeeEvent(source,action,argument));

    }

    public void setCurrentMessage(NNTPMessage message) {
        
    }
}
