/* 	BundleNewsGroup.java
 *	Copyright (c) 2009, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: May 27, 2009
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.plugin.search;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Pattern;

import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.MultiPartGroup;
import org.brains2b.coffee.message.NewsGroupImpl;
import org.brains2b.coffee.message.filter.MessageFilter;
import org.brains2b.coffee.message.local.Subscription;
import org.brains2b.log.Logger;

public class BundleNewsGroup extends NewsGroupImpl implements Cloneable {

	private Map m_filters;
	private Map m_articles;
	private String m_server;
	private int m_frequency;
	private boolean m_queued;
	private Pattern m_pattern;
	public BundleNewsGroup() {
		super();
	}
	
	
	@Override
	public void addLine(String s) {
		if (isHeaderSet() && getType()==224 && m_pattern.matcher(s).matches()) {
			Article a = new Article();
			addArticle(a);
		} else {
			super.addLine(s);
		}
	}
	@Override
	public void addArticle(Article a) {
		synchronized (m_filters) {
    		for (Iterator it = m_filters.keySet().iterator();it.hasNext();) {
    			String key = (String) it.next();
    			List filters = (List) m_filters.get(key);
    			boolean add= false;
    			for (Iterator itf = filters.iterator();itf.hasNext();) {
            		MessageFilter mf = (MessageFilter) itf.next();
            		if (mf.getKey().equals("new")) {
            			add = a.isNew();	
            		} else { 
            			Object value =a.getHeader().getHeaderObject(mf.getKey());
            			add = add || mf.isValid(value);
            		}
        		}
    			if (add) {
    				a.setNewsGroup(getName());
    				((MultiPartGroup) m_articles.get(key)).addArticle(a);
    			}
    		}
    		m_filters.notify();
		} 
	}
	
	public void removeFilters(String qryName) {
		m_articles.remove(qryName);
		m_filters.remove(qryName);
	}
	
	public void addFilters(Subscription group, String qryName, List filters, int freq, List current) {
		if (getName()==null) {
			setName(group.getName());
			setFirst(group.getFirst());
			setLast(group.getLast());
			setLastUpdated(group.getLastUpdated());
			setFrequency(freq);
			setServer(group.getServer());
		} else if (!isQueued()) {
			if (getFirst()>group.getFirst()) {
				setFirst(group.getFirst());
			}
			if (getLast()>group.getLast()) {
				setFirst(group.getLast());
			}
			if (getLastUpdated().after(group.getLastUpdated())) {
				setLastUpdated(group.getLastUpdated());
			}
			if (freq<getFrequency()) {
				setFrequency(freq);
			}
		}
		if (m_filters==null) {
			m_filters = new HashMap();
			m_articles = new HashMap();
		}
		MultiPartGroup mpg = new MultiPartGroup();
		mpg.setArticleList(current);
		synchronized (m_filters) {
			m_articles.put(qryName, mpg);
			m_filters.put(qryName, filters);
			m_filters.notify();
		}
		createPattern(filters);
	}
	
	public Set keySet() {
		return m_filters.keySet();
	}
	
	public boolean isEmpty() {
		return m_filters.isEmpty();
	}
	                         
	
	public List getArticleList(String qryName) {
		return ((MultiPartGroup) m_articles.get(qryName)).getArticleList();
	}

	public String getServer() {
		return m_server;
	}

	public void setServer(String server) {
		m_server = server;
	}

	@Override
	public Object clone() {
		BundleNewsGroup bng= new BundleNewsGroup();
		bng.m_filters = m_filters;
		bng.m_articles = m_articles;
		bng.setName(getName());
		bng.setFirst(getFirst());
		bng.setLast(getLast());
		bng.setLastUpdated(getLastUpdated());
		bng.setServer(getServer());
		return bng;
	}

	public int getFrequency() {
		return m_frequency;
	}

	public void setFrequency(int frequence) {
		m_frequency = frequence;
	}

	public boolean isQueued() {
		return m_queued;
	}

	public void setQueued(boolean queued) {
		m_queued = queued;
	}
	
	public void createPattern(List<MessageFilter> filters) {
		if (filters==null || filters.isEmpty()) return;
		StringBuffer sb = new StringBuffer();
		sb.append("(");
		if (m_pattern!=null) {
			String old = m_pattern.pattern();
			sb.append(old.substring(1,old.length()-2));
		}
		for (MessageFilter f:filters) {
			String r = f.toRegEx();
			if (r!=null) {
				if (sb.length()>1) sb.append("|");
				sb.append(r);
			}
		}
		sb.append(")");
		Logger.println("new pattern for " + getName() + ":"  + sb.toString(),Logger.DEBUG);
		m_pattern = Pattern.compile(sb.toString(),Pattern.CASE_INSENSITIVE);
		
		
	}
}
