/* 	NewsSearch.java
 *	Copyright (c) 2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Feb 25, 2008
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * 28-03-2008	dennis@brains2b.org		Processing could be started multiple times
 * 14-05-2008	dennis@brains2b.org		setDividerLocation correctly at startup
 * 17-05-2008	dennis@brains2b.org		Fixes for search group routine
 * 06-01-2009	dennis@brains2b.org		Fix for adding new queries		
 */
package org.brains2b.coffee.plugin.search;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.dnd.DropTarget;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.SpinnerNumberModel;

import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.file.FileConnector;
import org.brains2b.coffee.gui.CfDetailView;
import org.brains2b.coffee.gui.CfFilterRow;
import org.brains2b.coffee.gui.CfStatsDelegate;
import org.brains2b.coffee.gui.NewsGroupRenderer;
import org.brains2b.coffee.gui.dnd.DropNewsGroup;
import org.brains2b.coffee.gui.table.ArticleTable;
import org.brains2b.coffee.gui.table.TreeMaker;
import org.brains2b.coffee.gui.widget.NewIndicator;
import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.NewsGroup;
import org.brains2b.coffee.message.NewsGroupImpl;
import org.brains2b.coffee.message.filter.MessageFilter;
import org.brains2b.coffee.message.local.FileMessage;
import org.brains2b.coffee.plugin.CoffeeExchangeView;
import org.brains2b.coffee.plugin.MimePlugin;
import org.brains2b.file.PostFixFilter;
import org.brains2b.swingext.action.ExtActionMap;
import org.brains2b.swingext.dialog.DialogHelper;
import org.brains2b.swingext.gui.list.ExtMutableList;
import org.brains2b.task.TaskEvent;
import org.brains2b.task.TaskListener;

/**
 * NewsSearch GUI screen
 * 
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.90 [Feb 25, 2008]
 */
public class NewsSearch extends JPanel implements CoffeeExchangeView,
		MimePlugin, ActionListener, TaskListener {

	class ComboRenderer extends DefaultListCellRenderer {

		public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) {
			JLabel l = (JLabel) super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			if (value instanceof QueryMessage) {
    			QueryMessage m = (QueryMessage) value;
    			int cnt = m.getNewMessages();
    			if (cnt>0) {
    				l.setHorizontalTextPosition(JLabel.LEADING);
    				l.setIcon(new NewIndicator(cnt));
    			}
			}
			return l;
		}
	}
	
	class ComboHandler implements ItemListener {

		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.DESELECTED && m_current != null) {
				if (m_current.getFilters() == null
						|| m_current.getFilters().size() != m_qlist.getData()
								.size()) {
					m_current.setFilters(m_qlist.getData());
				}
				if (m_current.getNewsgroups() == null
						|| m_current.getNewsgroups().size() != m_glist
								.getData().size()) {
					m_current.setNewsgroups(m_glist.getData());
				}

			} else if (e.getStateChange() == ItemEvent.SELECTED) {
				Object o = m_queries.getSelectedItem();
				if (m_queries.isEditable()) {
					m_current.setId(o.toString());
					m_current.setFrequency(1);
					m_queries.setEditable(false);
				} else {
					m_current = (QueryMessage) o;

				}
				m_qlist.setData(m_current.getFilters());
				m_glist.setData(m_current.getNewsgroups());
				m_table.getArticleModel().setData(
						TreeMaker.order(m_current.getId(),m_current.getArticles()));
				m_stats.setCurrentMessage(m_current);
			}
		}
	}
	
	class MouseHandler extends MouseAdapter {
		
		@Override
		public void mouseClicked(MouseEvent e) {
			//TODO Should be an action
			if (e.getButton()==MouseEvent.BUTTON1 && e.getClickCount()>=2) {
				int idx = m_table.getSelectedRow();
				if (idx==-1) return;
				Article a = m_table.getArticleModel().getArticle(idx);
				CfDetailView view = new CfDetailView();
				Dimension d =DialogHelper.getWindowPane(NewsSearch.this).getSize();
				d.width = d.width/2;
				d.height = d.height/2;
				view.setPreferredSize(d);
				view.activate();
				fireCoffeeEvent(view, CoffeeEvent.GET_ARTICLE, a);
				DialogHelper.showDialog(NewsSearch.this, view, a.getHeader().getSubject());
				view.deactivate();
			}
		}
	}

	private JComboBox m_queries;
	private JSplitPane m_split;
	private ExtMutableList m_glist;
	private ExtMutableList m_qlist;
	private QueryMessage m_current;
	private CfStatsDelegate m_stats;
	private ArticleTable m_table;
	private NewsSearchProcess m_process;
	private int m_lastCnt;

	/**
	 * Constructor
	 */
	public NewsSearch() {
		super();
		init();
	}

	public void activate() {
		Dimension d = m_split.getSize();
		Dimension top = m_split.getTopComponent().getPreferredSize();
		m_split.setDividerLocation((double) (top.height + m_split
				.getDividerSize())
				/ (double) d.height);

	}

	public void deactivate() {
		if (m_current != null) {
			if (m_current.getFilters() == null
					|| m_current.getFilters().size() != m_qlist.getData()
							.size()) {
				m_current.setFilters(m_qlist.getData());
			}
			if (m_current.getNewsgroups() == null
					|| m_current.getNewsgroups().size() != m_glist.getData()
							.size()) {
				m_current.setNewsgroups(m_glist.getData());
			}
		}

	}

	public void fireCoffeeEvent(CoffeeExchange source, int action,
			Object argument) {
		TicketBroker.getBroker().broker(
				new CoffeeEvent(source, action, argument));

	}

	public JMenu[] getMenus() {
		return null;
	}

	public void setCurrentMessage(NNTPMessage message) {
		if (message instanceof FileMessage) {
			FileMessage fm = (FileMessage) message;
			((DefaultComboBoxModel) m_queries.getModel())
					.addElement(((QueryHandler) fm.getContent()).getMessage());
			m_process.addQueryMessage((QueryMessage) ((QueryHandler) fm
					.getContent()).getMessage());
			if (m_queries.getItemCount() > 1) {
				m_queries.setSelectedIndex(0);
			}
			if (m_process.isFinished()) {
				m_process.setFinished(false);
				fireCoffeeEvent(this, CoffeeEvent.PROCESS, m_process);
			}
		}
	}

	private void init() {
		ResourceBundle rb = ResourceBundle.getBundle("org.brains2b.coffee.resource.coffee");
		setLayout(new BorderLayout());
		setActionMap(new NewsSearchMap(this));
		JPanel header = new JPanel();
		m_queries = new JComboBox();
		m_queries.setToolTipText(rb.getString("Search.QueryName.Tooltip"));
		m_queries.setRenderer(new ComboRenderer());
		m_queries.addItemListener(new ComboHandler());
		m_queries.setModel(new DefaultComboBoxModel());
		m_queries.setPrototypeDisplayValue("comp.lang.java.gui");
		header.add(m_queries);
		JButton qnew = new JButton(getActionMap().get("query.new"));
		qnew.setPreferredSize(new Dimension(24, 24));
		header.add(qnew);
		JButton qdel = new JButton(getActionMap().get("query.delete"));
		qdel.setPreferredSize(new Dimension(24, 24));
		header.add(qdel);
		add(header, BorderLayout.NORTH);

		JPanel main = new JPanel(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.insets = new Insets(5, 5, 5, 5);
		gbc.fill = GridBagConstraints.BOTH;

		gbc.gridx = 1;
		gbc.gridy = 3;
		gbc.gridwidth = 3;
		main.add(new JLabel(rb.getString("Search.QueryList.Title")), gbc);
		m_qlist = new ExtMutableList();
		m_qlist.setPrototypeCellValue("Subject contains The alphabet");
		m_qlist.addActionListener(this);
		
		gbc.gridy++;
		gbc.gridheight = 8;
		main.add(m_qlist, gbc);

		gbc.gridx = 6;
		gbc.gridy = 3;
		gbc.gridheight = 1;
		main.add(new JLabel(rb.getString("Search.Newsgroups.Title")), gbc);
		m_glist = new ExtMutableList(ExtMutableList.ACTION_REMOVE);
		m_glist.addActionListener(this);
		m_glist.setDropTarget(new DropTarget(m_glist,
				new DropNewsGroup(m_glist)));
		NewsGroup ng = new NewsGroupImpl();
		ng.setName("comp.lang.java.programmer");
		m_glist.setPrototypeCellValue(ng);
		m_glist.setCellRenderer(new NewsGroupRenderer(0));
		gbc.gridy++;
		gbc.gridheight = 8;
		main.add(m_glist, gbc);

		gbc.gridy += 8;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		gbc.gridx = 1;
		main.add(new JLabel(rb.getString("Search.Frequency")), gbc);
		gbc.gridx++;
		JSpinner spinner = new JSpinner(new SpinnerNumberModel(1, 1, 100, 1));
		spinner.setToolTipText(rb.getString("Search.Frequency.Tooltip"));
		main.add(spinner, gbc);
		gbc.gridx++;
		main.add(new JLabel(rb.getString("Search.Frequency.Post")), gbc);

		// gbc.gridy++;
		gbc.gridx = 7;
		JButton qref = new JButton(getActionMap().get("query.refresh"));
		qref.setPreferredSize(new Dimension(24, 24));
		qref.setMaximumSize(new Dimension(24, 24));
		main.add(qref, gbc);

		m_split = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		add(m_split, BorderLayout.CENTER);
		m_split.setTopComponent(main);

		JPanel tblPane = new JPanel(new BorderLayout());
		m_table = new ArticleTable(true);
		ExtActionMap map = new ExtActionMap(this);
		map.addAction("save.attach", rb.getString("Article.SaveAttach"), null, null,
				"ctrl shift s", 's');
		map
				.addAction("save.message", rb.getString("Article.Save"), null, null,
						"ctrl s", 's');
		m_table.setActionMap(map);
		m_table.addMouseListener(new MouseHandler());
		tblPane.add(new JScrollPane(m_table), BorderLayout.CENTER);
		m_stats = new CfStatsDelegate(this);
		tblPane.add(m_stats.addStats(QueryMessage.class), BorderLayout.SOUTH);

		m_split.setBottomComponent(tblPane);
	}

	public String getShortTitle() {
		ResourceBundle rb = ResourceBundle.getBundle("org.brains2b.coffee.resource.coffee");
		return rb.getString("Search.Title");
	}

	public String getDescription() {
		ResourceBundle rb = ResourceBundle.getBundle("org.brains2b.coffee.resource.coffee");
		return rb.getString("Search.Description");
	}
	
	public boolean isActionSupported(String mime, int action) {
		return true;
	}

	public void actionPerformed(ActionEvent e) {
		String action = e.getActionCommand();
		if (action.equals("save.attach")) {
			int[] rows = m_table.getSelectedRows();
			for (int i = 0; i < rows.length; i++) {
				fireCoffeeEvent(this, CoffeeEvent.SAVE_ATTACHMENT, m_table
						.getArticleModel().getArticle(rows[i]));
			}
		} else if (action.equals("save.message")) {
			int[] rows = m_table.getSelectedRows();
			for (int i = 0; i < rows.length; i++) {
				fireCoffeeEvent(this, CoffeeEvent.SAVE_POST, m_table
						.getArticleModel().getArticle(rows[i]));
			}
		} else if ("query.new".equals(action)) {
			m_current=null;
			m_queries.addItem(new QueryMessage());
			m_queries.setSelectedIndex(m_queries.getItemCount()-1);
			m_queries.setEditable(true);
			m_queries.requestFocus();
		} else if ("query.delete".equals(action)) {
			QueryMessage qm = (QueryMessage) m_queries.getSelectedItem();
			if (qm != null) {
				m_queries.removeItem(qm);
				m_process.removeQueryMessage(qm);
				delete(qm);
			}
			
		} else if ("query.refresh".equals(action)) {
			QueryMessage qm = (QueryMessage) m_queries.getSelectedItem();
			if (qm != null) {
				m_process.removeQueryMessage(qm);
				qm.setDirect(true);
				m_process.addQueryMessage(qm);
				if (m_process.isFinished()) {
					fireCoffeeEvent(this, CoffeeEvent.PROCESS, m_process);
				}

			}
		} else if ("ExtMutableList.Add".equals(action)
				&& e.getSource() == m_qlist) {
			CfFilterRow filterRow = new CfFilterRow(true);
			DialogHelper.showDialog(this, filterRow, "Query");
			if (filterRow.getMessageFilter() != null) {
				MessageFilter mf = filterRow.getMessageFilter();
				mf.setNot(false);
				m_current.getFilters().add(mf);
				m_qlist.addItem(mf);
			}
		} else if ("ExtMutableList.Edit".equals(action)
				&& e.getSource() == m_qlist) {
			int index = m_qlist.getSelectedIndex();
			MessageFilter mf = (MessageFilter) m_qlist.getSelectedValue();
			mf.setNot(false);
			CfFilterRow filterRow = new CfFilterRow(mf, true);
			DialogHelper.showDialog(this, filterRow, "Query");
			mf = filterRow.getMessageFilter();
			if (mf != null) {
				mf.setNot(false);
				m_current.getFilters().set(index, mf);
				m_qlist.replaceItem(mf, index);
			}
		} else if ("ExtMutableList.Remove".equals(action)) {
			if (e.getSource() == m_qlist) {
				m_qlist.removeItem(m_qlist.getSelectedIndex());
			} else {
				m_glist.removeItem(m_glist.getSelectedIndex());
			}
		}
	}

	private void load() {
		File[] dir = new File(FileConnector.getSystemPath(), "query")
				.listFiles(new PostFixFilter("*.xml", "XML-files"));
		if (dir == null)
			return;
		for (int i = 0; i < dir.length; i++) {
			FileMessage fm = new FileMessage(dir[i], "text/xml.query");
			fireCoffeeEvent(this, CoffeeEvent.XML_READ, fm);
		}
		
	}

	private void save() {
		for (int i = 0; i < m_queries.getItemCount(); i++) {
			QueryMessage qm = (QueryMessage) m_queries.getItemAt(i);
			if (qm != null) {
				fireCoffeeEvent(this, CoffeeEvent.XML_SAVE, qm);
			}
		}
	}

	public void delete(QueryMessage qm) {
		//TODO delete should be an activity of FileConnector
		FileConnector fc = (FileConnector) TicketBroker.getBroker().getTicketHandler(CoffeeEvent.XML_READ);
		File f = new File(new File(fc.getPath().getParent(),"query"), qm.getId()+ ".xml");
		f.delete();
	}

	public int getSelectEvent(int messageType) {
		return CoffeeEvent.NONE;
	}

	public void close() {
		m_process.setClosing(true);
		save();
	}

	public void open() {
		m_process = new NewsSearchProcess();
		m_process.addTaskListener(this);
		load();
	}

	public void taskProgressed(TaskEvent e) {
		QueryMessage qm = (QueryMessage) e.getSource();
		if (qm == m_current) {
			if (qm.getArticles() != null
					&& qm.getArticles().size() != m_table.getArticleModel()
							.getRowCount()) { // m_articleModel counts root as
												// well
				List l = getSelectedArticles();
				m_table.getArticleModel().setData(
						TreeMaker.order(qm.getId(),qm.getArticles()));
				setSelectedArticles(l);
				m_stats.setCurrentMessage(qm);
			}
		}
		updateForNew(qm);
	}

	private void updateForNew(QueryMessage msg) {
		int newcnt = 0;
		for (int i=0;i<m_queries.getItemCount();i++) {
			newcnt += ((QueryMessage) m_queries.getItemAt(i)).getNewMessages();
		}
		if (newcnt > 0 && newcnt != m_lastCnt) {
			m_lastCnt = newcnt;
			createIcon(getTabPage(this), newcnt);
		}
	}

	private JTabbedPane getTabPage(JComponent c) {
		if (c.getParent() instanceof JTabbedPane) {
			return (JTabbedPane) c.getParent();
		} else {
			return getTabPage((JComponent) c.getParent());
		}
	}

	private void createIcon(JTabbedPane tab, int newcnt) {
		if (tab == null)
			return;
		tab.setIconAt(tab.indexOfComponent(this), new NewIndicator(newcnt));
	}
	
	private List<Article> getSelectedArticles() {
		int count = m_table.getSelectedRowCount();
		List<Article> l = new ArrayList<Article>();
		if (count==0) return null;
		for (int i=0;i<count;i++) {
			l.add(m_table.getArticleModel().getArticle(m_table.getSelectedRows()[i]));
		}
		return l;
	}
	
	private void setSelectedArticles(List<Article> l) {
		if (l==null) return;
		for (int i=0;i<m_table.getArticleModel().getRowCount();i++) {
			if (l.contains(m_table.getArticleModel().getArticle(i))) {
				m_table.addRowSelectionInterval(i, i);
			}
		}
	}
}
