/* 	NewsSearchProcess.java
 *	Copyright (c) 2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Feb 25, 2008
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * 28-03-2008	dennis@brains2b.org		Fixed calculation of first and last for getArticles()
 * 09-04-2008	dennis@brains2b.org		findNewsGroup should also check previous queries	
 * 17-05-2008	dennis@brains2b.org		Fixes for search group routine	
 * 06-01-2009	dennis@brains2b.org		Don't call cleanup on closing
 */
package org.brains2b.coffee.plugin.search;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;

import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.MultiPartGroup;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.NewsGroup;
import org.brains2b.coffee.message.local.CoffeeMenu;
import org.brains2b.coffee.message.local.ServerDescriptor;
import org.brains2b.coffee.message.local.Subscription;
import org.brains2b.coffee.plugin.CoffeeExchangeControl;
import org.brains2b.log.Logger;
import org.brains2b.task.Task;
import org.brains2b.task.TaskEvent;
import org.brains2b.task.TaskListener;

/** Main Processor for Queries
 * <p>Runs query messages by calling articles list 25.000 at a time 
 * for the newsgroups in this query one by one. FilterInNewsGroup is
 * used to filter in only the articles matching. All others will be ignored
 * to save memory.
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.90.1 [09-04-2008]
 * @see FilterInNewsGroup
 */
public class NewsSearchProcess implements CoffeeExchangeControl, TaskListener {

	private final static int MAX = 25000;
	
	private boolean m_finished =true;
	private boolean m_closing = false;
	private int m_taskSpawned =0;
	private Date m_thisDate;
	private ServerDescriptor m_server;
	private String m_findName;
	private ArrayList m_listeners;
	private QueryBundle m_queries;
	private Object m_mutex;
	
	/**
	 * Constructor
	 *
	 */
	public NewsSearchProcess() {
		super();
		m_queries = new QueryBundle();
		m_mutex = this;	
	}
	
	public TaskListener getTaskListener() {
		return this;
	}

	public Stack getMessages() {
		return null;
	}

	public int getResult() {
		return RESULT_OK;
	}

	public boolean isFinished() {
		return m_finished;
	}

	public void setFinished(boolean b) {
		m_finished = b;
	}

	public void run() {
		Logger.println("WaarO?");
		setFinished(false);
		m_thisDate = new Date();
		synchronized (m_mutex) {
			for (Iterator it = m_queries.groupIterator(); it.hasNext(); ) {
				BundleNewsGroup bng = (BundleNewsGroup) it.next();
				Calendar c= Calendar.getInstance();
				c.roll(Calendar.DAY_OF_YEAR, -bng.getFrequency());
				if (bng.getLastUpdated().before(c.getTime())) {
					findServer(bng.getServer());
					bng.setQueued(true);
					bng.setFirst(bng.getLast());
					bng.setLast(bng.getLast()+MAX);
					m_taskSpawned++;
					TicketBroker.getBroker().broker(
								new CoffeeEvent(this,CoffeeEvent.GET_ARTICLE_LIST,bng)
							);
				}
				
			}
		} 
		setFinished(true);
	}

	public void setCurrentMessage(NNTPMessage message) {
		if (message instanceof NewsGroup) {
			synchronized (m_mutex) {
			BundleNewsGroup ng = null;
			
			if (message instanceof MultiPartGroup) {
				ng = (BundleNewsGroup) ((MultiPartGroup) message).getDelegate();
			} else {
				ng = (BundleNewsGroup) message;
			}
			
			for (Iterator it = ng.keySet().iterator();it.hasNext();) {
				String key = (String) it.next();
				QueryMessage qm = m_queries.getQueryMessage(key);
				/*if (qm.getArticles()==null) {
					qm.setArticles(new ArrayList());
				} 
				List l = ng.getArticleList(key);
				if (l!=null) {
					for( Iterator itl = l.iterator();itl.hasNext();) {
						Article a = (Article) itl.next();
						a.setNewsGroup(ng.getName());
						if (!qm.getArticles().contains(a)) {
							qm.addArticle(a);
						}
					}
					l.clear();
				}*/
				
				Subscription grp = findSubscription(qm,ng.getName());
				if (grp==null) {
					Logger.print(ng.getName() + "could not be found", Logger.ERROR);
					return;
				}
				findServer(grp.getServer());
				if (isClosing()) return;
				if (grp!=null) {
					
					if (grp.getLastUpdated().before(m_thisDate)) {
						grp.setFirst(ng.getFirst());
						grp.setLastUpdated(m_thisDate);
					}
					
					if (ng.getLast()-ng.getFirst()<MAX) {
						grp.setLast(ng.getLast());
						
					} 
					
					fireTaskEvent(new TaskEvent(qm,0,0,qm.getNewsgroups().indexOf(grp),qm.getNewsgroups().size()));
				}
			}
			if (ng.getLast()-ng.getFirst()>=MAX) {
				ng.setFirst(ng.getLast());
				ng.setLast(ng.getLast()+MAX);
				TicketBroker.getBroker().broker(
					new CoffeeEvent(this,CoffeeEvent.GET_ARTICLE_LIST,ng)
				);
				m_taskSpawned--;
			} else {
				if (!isClosing()) cleanup(ng);
				ng.setQueued(false);
				fireListUpdated(ng);
			}
			
			m_mutex.notify();
			}	
		} else if ( message instanceof CoffeeMenu && message.getId().equals("server")) {
   			for (Iterator it = ((CoffeeMenu) message).getList().iterator();it.hasNext();) {
   				ServerDescriptor sd = (ServerDescriptor) it.next();
   				if (m_findName.equals(sd.getId())) {
   					m_server = sd;
   					break;
   				}
   			}
		}
		
	}

	/**
	 * add a QueryMessage to this process
	 * <p>Works on both finished and running process
	 * @param qm {@link QueryMessage} to add
	 */
	public void addQueryMessage(QueryMessage qm) {
		synchronized (m_mutex) {
			m_queries.addQueryMessage(qm);
			m_mutex.notify();
		}
	}
	
	/**
	 * remove a QueryMessage from this process
	 * <p>Works on running process, but not if this
	 * is the current one running
	 * @param qm {@link QueryMessage} to add
	 */
	public void removeQueryMessage(QueryMessage qm) {
		synchronized (m_mutex) {
			m_queries.removeQueryMessage(qm);
			m_mutex.notify();
		}
		
	}
	
	public List getQueryMessages() {
		if (m_queries==null) return null;
		return m_queries.getQueries();
	}

	public void taskProgressed(TaskEvent e) {
		// decrease taskSpawned only if there is no current message
		if (e.isFinished() && ((Task) e.getSource()).getResult()!=Task.RESULT_OK) {
			m_taskSpawned--;
		}
		
	}
	
	/**
	 * add a TaskListener to this process
	 * @param tl TaskListener
	 */
	public void addTaskListener(TaskListener tl) {
		if (m_listeners==null) {
			m_listeners=new ArrayList();
		}
		m_listeners.add(tl);
	}

	/**
	 * remove a TaskListener from this process
	 * @param tl TaskListener
	 */
	public void removeTaskListener(TaskListener tl) {
		if (m_listeners!=null) {
		    m_listeners.remove(tl);
		}
	}
	
	private void fireListUpdated(BundleNewsGroup bng) {
		for (Iterator it = bng.keySet().iterator();it.hasNext();) {
			String key = (String) it.next();
			QueryMessage qm = (QueryMessage) m_queries.getQueryMessage(key);
			TaskEvent te = new TaskEvent(qm,0,0,1,qm.getNewsgroups().size());
			te.setFinished(true);
			fireTaskEvent(te);
		}	
	}
	
	private void fireTaskEvent(TaskEvent e) {
	    if (m_listeners!=null) {
	        Iterator it=m_listeners.iterator();
	        while (it.hasNext()) {
	            ((TaskListener) it.next()).taskProgressed(e); 
	        }
	    }
	}
	
	private Subscription findSubscription(QueryMessage msg, String groupName) {
		for (Iterator it = msg.getNewsgroups().iterator();it.hasNext();) {
			Subscription s = (Subscription) it.next();
			if (groupName.equals(s.getId())) return s;
		}
		return null;
	}
	
	private synchronized ServerDescriptor findServer(String name) {
		if (m_server!=null && name.equals(m_server.getId())) {
			return m_server;
		}
		m_server = TicketBroker.getBroker().getCurrentServer();
		if (m_server!=null && name.equals(m_server.getId())) {
			return m_server;
		}
		m_server = null;
		m_findName = name;
		TicketBroker.getBroker().broker(
			new CoffeeEvent(this,CoffeeEvent.GET_LIST,"server")		
		);
		while (m_server==null) {
			if (isClosing()) return null;
			try {
				Thread.sleep(500);
			} catch (Exception ex) {}
		}
		TicketBroker.getBroker().broker(
			new CoffeeEvent(this,CoffeeEvent.CHANGE_SERVER,m_server)		
		);
		return m_server;
	}
	
	private void cleanup(BundleNewsGroup bng) { 
		for (Iterator it = bng.keySet().iterator();it.hasNext();) {
			String key = (String) it.next();
			QueryMessage qm = m_queries.getQueryMessage(key);
			List l =qm.getArticles();
			Subscription s = findSubscription(qm, bng.getName());
			if (l!=null)
			for (Iterator itl= l.iterator();itl.hasNext();) {
				Article a = (Article) itl.next();
				if (a.getNewsGroup()!=null && a.getNewsGroup().equals(s.getId()) && (a.getNumber()<s.getFirst() || a.getNumber()>s.getLast())) {
					itl.remove();
				}
			}
		}
	}

	public boolean isClosing() {
		return m_closing;
	}

	public void setClosing(boolean closing) {
		m_closing = closing;
	}
}
