/* 	QueryBundle.java
 *	Copyright (c) 2009, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: May 27, 2009
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.plugin.search;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.brains2b.coffee.message.NewsGroup;
import org.brains2b.coffee.message.local.Subscription;

public class QueryBundle {

	private Map m_groups;
	private List m_queries;
	
	public QueryBundle() {
		m_groups = new HashMap();
		m_queries = new ArrayList();
	}
	
	public void addQueryMessage(QueryMessage msg) {
		m_queries.add(msg);
		for (Iterator it = msg.getNewsgroups().iterator();it.hasNext();) {
			Subscription group = (Subscription) it.next();
			BundleNewsGroup bundle = (BundleNewsGroup) m_groups.get(group.getName());
			if (bundle == null) {
				bundle = new BundleNewsGroup();
				m_groups.put(group.getName(), bundle);
			}
			if (msg.getArticles()==null) {
				msg.setArticles(new ArrayList());
			}
			bundle.addFilters(group, msg.getId(), msg.getFilters(), msg.isDirect()?0:msg.getFrequency(),msg.getArticles());
		}
	}
	
	public void removeQueryMessage(QueryMessage msg) {
		m_queries.remove(msg);
		for (Iterator it = msg.getNewsgroups().iterator();it.hasNext();) {
			NewsGroup group = (NewsGroup) it.next();
			BundleNewsGroup bundle = (BundleNewsGroup) m_groups.get(group.getName());
			if (bundle!=null) {
				bundle.removeFilters(msg.getId());
				if (bundle.isEmpty()) {
					it.remove();
				}
			}
		}
	}
	
	
	public QueryMessage getQueryMessage(String qryId) {
		for (Iterator it = m_queries.iterator();it.hasNext();) {
			QueryMessage qm = (QueryMessage) it.next();
			if (qryId.equals(qm.getId())) {
				return qm;
			}
		}
		return null;
	}
	
	public Iterator groupIterator() {
		ArrayList l = new ArrayList();
		for (Iterator it = m_groups.values().iterator();it.hasNext();) {
			BundleNewsGroup bng = (BundleNewsGroup) it.next();
			if (!bng.isQueued()) l.add(bng);
		}
		return l.iterator();
	}

	public List getQueries() {
		return m_queries;
	}
	
	
}
