/* 	CfDateHelper.java
 *	Copyright (c) 2005-2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 27-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * May 8, 2007	dennis@brains2b.nl      Return null if Date is null
 * 29-02-2008	dennis@brains2b.nl		Method for parse is faster if String is split beforehand
 * 09-04-2008	dennis@brains2b.org		Synchronize format/parse while SimpleDateFormat is not		
*/
package org.brains2b.coffee.util;

import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.brains2b.log.Logger;

/** Helper functions for Dates formatted from Newsgroup
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.89.2 [09-04-2008]
 */
public class CfDateHelper {
	
	private final static String SHORT="d MMM yyyy HH:mm:ss Z";
	private final static String LONG ="EEE, d MMM yyyy HH:mm:ss Z";
	
	private static SimpleDateFormat m_parse=new SimpleDateFormat(SHORT,Locale.US);
    private static SimpleDateFormat m_format=new SimpleDateFormat(LONG,Locale.US);
	
	/**
	 * parse a String as Date in newsgroup format
	 * @param s String, String containing a Date
	 * @return Date, if the text could not be parsed the
	 * date for today is returned
	 */
	public final static Date parse(String s) {
	    final int idx = s.indexOf(',');
	    synchronized (m_parse) {
			
			ParsePosition p = new ParsePosition(0);
		    if (idx!=-1) {
		    	p.setIndex(idx+2);
		    }
	       try {
	    		return m_parse.parse(s,p);
	       } catch (Exception ex) {
	    	   Logger.printException(ex);
	    	   return new Date();
	       }
	    }
        
	}
	
	/**
	 * Format a date as <code>EEE, d MMM yyyy HH:mm:ss Z</code>
	 * @param d Date, the date to format
	 * @return String, null if date is null or could not be formatted
	 */
	public final static String format(Date d) {
        if (d==null) return null;
        try {
        	synchronized (m_format) {
	        	String s = m_format.format(d);
		    	return s;
        	}
        } catch (Exception ex) {
        	return null;
        }
	}
}
