package test.org.brains2b.coffee.message.filter;

import java.io.File;
import java.io.StringWriter;

import junit.framework.TestCase;

import org.brains2b.coffee.message.filter.FilterList;
import org.brains2b.coffee.message.filter.MessageFilter;
import org.brains2b.liken.LikenException;
import org.brains2b.liken.xml.XmlCompare;
import org.brains2b.saxo.SAXWriter;
import org.brains2b.util.DateHelper;
import org.xml.sax.SAXException;

public class FilterListTest extends TestCase {

	protected void setUp() throws Exception {
		super.setUp();
	}

	public void testFilterList() throws SAXException, LikenException{
		FilterList fl = FilterList.getInstance();
		assertEquals(fl.getSize(),0);
		assertFalse(fl.hasContent());
		
		MessageFilter allS = new MessageFilter("Subject", "contains","Test");
		MessageFilter allD = new MessageFilter("Date", "lesser",DateHelper.isoToDate("2010-12-30"));
		MessageFilter tstS = new MessageFilter("Subject", "Test");
		
		fl.addFilter(FilterList.ALL, allS);
		fl.addFilter(FilterList.ALL, allD);
		fl.addFilter("alt.test", tstS);
		
		assertEquals(3,fl.getSize());
		assertTrue(fl.hasContent());
		
		assertEquals(3,fl.getAllFilters("alt.test").size());
		assertEquals(2,fl.getAllFilters("alt.x").size());
		
		assertEquals(2,fl.getFilterNames().size());
		assertTrue(fl.getFilterNames().contains("alt.test"));
		
		assertTrue(fl.getFilters(FilterList.ALL).contains(allS));
		assertTrue(fl.getFilters(FilterList.ALL).contains(allD));
		assertTrue(!fl.getFilters(FilterList.ALL).contains(tstS));
		assertTrue(fl.getFilters("alt.test").contains(tstS));
		
		StringWriter buf = new StringWriter();
		SAXWriter sw = new SAXWriter(buf);
		fl.writeMessage(sw);
		XmlCompare xml = new XmlCompare();
		boolean b = xml.compare(new File("test/filters.xml"), buf.toString());
		assertTrue(xml.getReport(), b);
		
	}
}
