/* 	SubjectHelperTest.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 10-aug-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package test.org.brains2b.coffee.message.filter;

import junit.framework.TestCase;

import org.brains2b.coffee.util.SubjectHelper;
import org.brains2b.coffee.util.SubjectHelper.SubjectInfo;

public class SubjectHelperTest extends TestCase {
   
    public SubjectHelperTest(String arg0) {
        super(arg0);
    }

    public void testTrucatedBasic() {
        String test="Parting is such a sweet sorrow (1/5)";
        SubjectHelper sh = new SubjectHelper();
        SubjectInfo info =sh.getTruncated(test);
        assertNotNull(info);
        assertEquals("Parting is such a sweet sorrow",info.subject);
        assertEquals(1,info.part);
        assertEquals(5,info.total);
    }
    
    public void testTrucatedBlock() {
        String test="Parting [0/100] is such a sweet sorrow (1/5)";
        SubjectHelper sh = new SubjectHelper();
        SubjectInfo info =sh.getTruncated(test);
        assertNotNull(info);
        assertEquals("Parting [0/100] is such a sweet sorrow",info.subject);
        assertEquals(1,info.part);
        assertEquals(5,info.total);
    }
    
    public void testTrucatedTwice() {
        String test="Parting (0/100) is such a sweet sorrow (1/5)";
        SubjectHelper sh = new SubjectHelper();
        SubjectInfo info =sh.getTruncated(test);
        assertNotNull(info);
        assertEquals("Parting (0/100) is such a sweet sorrow",info.subject);
        assertEquals(1,info.part);
        assertEquals(5,info.total);
    }
    
    public void testTrucatedOther() {
        String test="Parting ( garbage/collection ) is such a sweet sorrow (1/5)";
        SubjectHelper sh = new SubjectHelper();
        SubjectInfo info =sh.getTruncated(test);
        assertNotNull(info);
        assertEquals("Parting ( garbage/collection ) is such a sweet sorrow",info.subject);
		assertEquals(1,info.part);
        assertEquals(5,info.total);
        }
    
    public void testTrucatedOtherEnd() {
        String test="Parting is such a sweet sorrow (1/5) ( garbage/collection )";
        SubjectHelper sh = new SubjectHelper();
        SubjectInfo info =sh.getTruncated(test);
        assertNotNull(info);
        assertEquals("Parting is such a sweet sorrow ( garbage/collection )",info.subject);
        assertEquals(1,info.part);
        assertEquals(5,info.total);
    }
    
    public void testTruncated() {
        String test ="heiligegeneraal [000/109] - \"heiligegeneraal.nzb\" yEnc (2/2)" ;
        SubjectHelper sh = new SubjectHelper();
        SubjectInfo info =sh.getTruncated(test);
        assertNotNull(info);
        assertEquals("heiligegeneraal [000/109] - \"heiligegeneraal.nzb\" yEnc",info.subject);
        assertEquals(2,info.part);
        assertEquals(2,info.total);
    }
    
    public void testWrong() {
    	String test = "(Najib Amhali (2008)) [21/30] - \"Najib Amhali Zorg (2008) DVDR Xvid.part20.rar\" yEnc (1/20)";
    	 SubjectHelper sh = new SubjectHelper();
        SubjectInfo info =sh.getTruncated(test);
        assertNotNull(info);
        assertEquals("(Najib Amhali (2008)) [21/30] - \"Najib Amhali Zorg (2008) DVDR Xvid.part20.rar\" yEnc",info.subject);
        assertEquals(1,info.part);
        assertEquals(20,info.total);
    }
}
