package test.org.brains2b.coffee.message.local;

import java.io.File;
import java.io.StringWriter;

import org.brains2b.coffee.message.local.ServerDescriptor;
import org.brains2b.liken.LikenException;
import org.brains2b.liken.xml.XmlCompare;
import org.brains2b.saxo.SAXWriter;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

import junit.framework.TestCase;

public class ServerDescriptorTest extends TestCase {

	private final static String SERVER = "news.server.com";
	private final static int PORT = 199;
	private final static String PATH = "/tmp";
	
	protected void setUp() throws Exception {
		super.setUp();
	}

	public void testServerDescription() throws SAXException, LikenException {
		ServerDescriptor sd = new ServerDescriptor();
		sd.setAddress(SERVER);
		sd.setPort(PORT);
		sd.setPath(PATH);
		
		assertEquals(ServerDescriptor.SERVER, sd.getType());
		assertEquals(SERVER, sd.getAddress());
		assertEquals(PORT, sd.getPort());
		assertEquals(PATH, sd.getPath());
		
		//defaults
		assertFalse(sd.isBinary());
		assertEquals(0, sd.getMaxConnections());
		assertNull(sd.getDownload());
		assertNull(sd.getUsername());
		assertNull(sd.getPassword());
		
		sd.setBinary(true);
		sd.setMaxConnections(5);
		sd.setDownload(PATH);
		sd.setUsername("test");
		sd.setPassword("kees");
		
		assertTrue(sd.isBinary());
		assertEquals(5, sd.getMaxConnections());
		assertEquals(PATH,sd.getDownload());
		assertEquals("test",sd.getUsername());
		assertEquals("kees",sd.getPassword());
		
		StringWriter buf = new StringWriter();
		SAXWriter sw = new SAXWriter(buf);
		sd.writeMessage(sw);
		System.out.println(buf);
		XmlCompare xml = new XmlCompare();
		boolean b = xml.compare(buf.toString(), new File("test/serverdescriptor.xml"));
		assertTrue(xml.getReport(),b);
	}

	public void testServerDescriptionArgs() {
		ServerDescriptor sd = new ServerDescriptor(SERVER,PORT,PATH);
		
		assertEquals(1001, sd.getType());
		assertEquals(SERVER, sd.getAddress());
		assertEquals(PORT, sd.getPort());
		assertEquals(PATH, sd.getPath());
		
		// NNTP defaults;
		assertEquals(1, sd.getSize());
		assertFalse(sd.isHeaderSet());
		assertTrue(sd.hasContent());
	}
	
	public void testServerFromXml() {
		ServerDescriptor sd = new ServerDescriptor();
		
		AttributesImpl atts = new AttributesImpl();
		atts.addAttribute(null, null, "address", "CDATA", SERVER);
		atts.addAttribute(null, null, "port", "CDATA", String.valueOf(PORT));
		atts.addAttribute(null, null, "maxConnections", "CDATA", String.valueOf(5));
		atts.addAttribute(null, null, "binary", "CDATA", String.valueOf(Boolean.TRUE));
		sd.setAttributes(atts);
		assertEquals(ServerDescriptor.SERVER, sd.getType());
		assertEquals(SERVER, sd.getAddress());
		assertEquals(PORT, sd.getPort());
		assertTrue(sd.isBinary());
		assertEquals(5, sd.getMaxConnections());
		
		sd.setContent("path", null, PATH);
		sd.setContent("download", null, PATH);
		sd.setContent("username", null, "test");
		sd.setContent("password", null, "kees");
		
		assertEquals(PATH,sd.getPath());
		assertEquals(PATH,sd.getDownload());
		assertEquals("test",sd.getUsername());
		assertEquals("kees",sd.getPassword());
	}
}
