package test.org.brains2b.coffee.plugin;

import java.awt.event.FocusListener;
import java.io.File;
import java.util.Stack;

import javax.swing.JMenu;

import org.brains2b.coffee.CoffeeSpill;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.local.FileMessage;
import org.brains2b.coffee.plugin.CoffeeExchangeView;
import org.brains2b.coffee.plugin.MimeHandler;
import org.brains2b.coffee.plugin.MimePlugin;
import org.brains2b.coffee.plugin.TargetContainer;

import junit.framework.TestCase;

public class MimeHandlerTest extends TestCase {

	class TargetContainerImpl implements TargetContainer {

		public void showPlugin(String id) {
			assertEquals("test", id);
			
		}

		public void registerPlugin(String id, CoffeeExchangeView view,
				String description) {
			assertEquals("test", id);
			
		}
		
	}
	
	class CEViewImpl implements CoffeeExchangeView, MimePlugin {

		private Stack m_callStack= new Stack();
		
		public void setCurrentMessage(NNTPMessage message) {
			m_callStack.add("setCurrentMessage");
		}

		public void fireCoffeeEvent(CoffeeExchange source, int action,
				Object argument) {
			
		}

		public JMenu[] getMenus() {
			return null;
		}

		public int getSelectEvent(int messageType) {
			return 0;
		}

		public void activate() {
			
		}

		public void deactivate() {
			
		}

		public boolean isVisible() {
			return false;
		}

		public void addFocusListener(FocusListener f) {
			
		}

		public void removeFocusListener(FocusListener f) {
			
		}

		public boolean isActionSupported(String mime, int eventID) {
			m_callStack.add("isActionSupported");
			if ("text/test".equals(mime)) {
				return true;
			} else if ("text/test-this".equals(mime) && eventID==CoffeeEvent.PROCESS) {
				return true;
			}
			return false;
		}

		public String getShortTitle() {
			m_callStack.add("getShortTitle");
			return "test";
		}

		public void open() {
			m_callStack.add("open");
			
		}

		public void close() {
			m_callStack.add("close");
			
		}

		public String getDescription() {
			m_callStack.add("getDescription");
			return getClass().getName();
		}

		public Stack getCallStack() {
			return m_callStack;
		}
		
	}
	
	public void testHandler() {
		MimeHandler handler = new MimeHandler(new TargetContainerImpl());
		assertTrue(handler.isAvailable()); //always true;
		assertEquals(1,handler.getHandledEvents().length);
		assertEquals(CoffeeEvent.USE, handler.getHandledEvents()[0]);
	}
	
	public void testExchangeView() throws CoffeeSpill {
		MimeHandler handler = new MimeHandler(new TargetContainerImpl());
		CEViewImpl view = new CEViewImpl();
		handler.registerPlugin(view);
		assertEquals("getDescription", view.getCallStack().pop());
		assertEquals("getShortTitle", view.getCallStack().pop());
		
		handler.open();
		assertEquals("open",view.getCallStack().pop());
		
		FileMessage fm = new FileMessage(new File("test.coffee"), "text/test"); //File is not accessed so does not have to exist
		handler.handleEvent(new CoffeeEvent(view,CoffeeEvent.PROCESS,fm));
		assertEquals("setCurrentMessage",view.getCallStack().pop());
		assertEquals("getShortTitle", view.getCallStack().pop()); // used to lookup the plugin in the TargetContainer
		assertEquals("isActionSupported",view.getCallStack().pop());
		
		
		fm = new FileMessage(new File("test.coffee"), "text/test-this"); 
		handler.handleEvent(new CoffeeEvent(view,CoffeeEvent.PROCESS,fm));
		assertEquals("setCurrentMessage",view.getCallStack().pop()); 
		assertEquals("getShortTitle", view.getCallStack().pop()); // used to lookup the plugin in the TargetContainer
		assertEquals("isActionSupported",view.getCallStack().pop());
		
		fm = new FileMessage(new File("test.coffee"), "text/html"); 
		handler.handleEvent(new CoffeeEvent(view,CoffeeEvent.PROCESS,fm)); 
		assertEquals("isActionSupported",view.getCallStack().pop());
		
		try {
			handler.handleEvent(new CoffeeEvent(view,CoffeeEvent.PROCESS,null));
			fail();
		} catch (IllegalArgumentException ilex) {
			assertTrue(true);
		}
		
	}
}
